# encoding: utf8
"Util functions for the Issak Buch project"


import re
import cgi

import Sort
import gen.lib
import Utils

from gen.display.name import displayer as _nd
import DateHandler

class MyDateHandler(DateHandler._Date_de.DateDisplayDE):
    def __init__(self):
        DateHandler._Date_de.DateDisplayDE.__init__(self,4)

    def _display_gregorian(self, date_val):
        year = self._slash_year(date_val[2], date_val[3])
        if date_val[0] and date_val[1]:
            return "%d.%d.%s" % (date_val[0], date_val[1], year)
        else:
            return DateHandler._Date_de.DateDisplayDE._display_gregorian(self, date_val)

dd = MyDateHandler()

cchars=['1','2','3','4','5','6','7','8','9','a','b',
        'c','d','e','f','g','h','i','j','k','l','m',
        'n','o','p','q','r','s','t','u','v','w','x',
        'y','z']

class Descendants:

    def __init__(self,database,daddy):
        assert database
        assert daddy
        self.database = database
        self.people = []
        self.doubles = {}
        queue  = [ (daddy.get_handle(),"1") ]
        bd_sort = Sort.Sort(self.database).by_birthdate
        lookup = {}
        while queue:
            handle, number = queue.pop(0)

            if handle in lookup:
                self.doubles[number] = lookup[handle]
                self.people.append( (None, number) )
            else:
                childhandles = []
                person = self.database.get_person_from_handle(handle)
                for family_handle in person.get_family_handle_list():
                    family = self.database.get_family_from_handle(family_handle)
                    childhandles.extend([ref.ref for ref in family.get_child_ref_list()])

                # assume children to be sorted
                #childhandles.sort(bd_sort)

                for c, child_handle in enumerate(childhandles):
                    queue.append( (child_handle, number+cchars[c]) )

                self.people.append( (handle, number) )
                lookup[handle] = number

        self.people_cf = list(self.people)
        self.people_cf.sort(None,lambda (h,n):n)

    def __len__(self):
        return len(self.people)
    
    def __iter__(self):
        return iter(self.people)

    def child_first(self):
        return self.people_cf

def cmp_num(n1, n2):
    if len(n1) < len(n2):
        return -1
    elif len(n1) > len(n2):
        return 1
    else:
        return cmp(n1, n2)

def sort_events(events):
    def mycmp((t1,e1),(t2,e2)):
        if e1.get_date_object().get_start_date() != gen.lib.Date.EMPTY and \
           e2.get_date_object().get_start_date() != gen.lib.Date.EMPTY:
            return cmp(e1.get_date_object(), e2.get_date_object())
        else:
            for t in ('geb','taufe','heirat','scheidung','tod','gefallen','vermisst','beerdigung'):
                if t1 == t:
                    return -1
                elif t2 == t:
                    return 1
            return 0
    events.sort(mycmp)

# Latex-Encode
def le(str):
    str = re.sub(r'"\b', r'\\glqq{}', str)
    str = re.sub(r'\b"', r'\\grqq{}', str)
    str = re.sub(r' km\b', r'\\,km', str)
    str = re.sub(r'[\.,!\?]"', r'\\grqq{}', str)
    str = re.sub(r'"', r'\\textquotedblleft{}', str)
    str = re.sub(r'(\[[0-9a-e ]+\])', r'\\numberreference{\1}', str)
    str = str.replace(u'−','--')
    str = str.replace(u'&','\\&')
    str = str.replace("<u>",'\\rufname{')
    str = str.replace("</u>","}")
    str = re.sub(r'(http://[\w\d/\._-]+)',r'\\url{\1}', str)
    str = str.replace("-)","\"~)")
    str = str.replace("S. ","S.~")
    str = re.sub(r'\)([a-zA-Z])',r')""\1',str)
    str = str.replace("-/","\"~/")
    str = str.replace("-","\"=")
    str = str.replace("/","\\slash{}")
    str = re.sub(r'\\url{([^}]*)"=',r'\\url{\1-',str)
    str = re.sub(r'\\url{([^}]*)\\slash{}',r'\\url{\1/',str)
    str = re.sub(r'\\url{([^}]*)"=',r'\\url{\1-',str)
    str = re.sub(r'\\url{([^}]*)\\slash{}',r'\\url{\1/',str)
    str = re.sub(r'\\url{([^}]*)"=',r'\\url{\1-',str)
    str = re.sub(r'\\url{([^}]*)\\slash{}',r'\\url{\1/',str)
    str = re.sub(r'\\url{([^}]*)"=',r'\\url{\1-',str)
    str = re.sub(r'\\url{([^}]*)\\slash{}',r'\\url{\1/',str)
    str = re.sub(r'\\url{([^}]*)"=',r'\\url{\1-',str)
    str = re.sub(r'\\url{([^}]*)\\slash{}',r'\\url{\1/',str)
    str = re.sub(r'\\url{([^}]*)"=',r'\\url{\1-',str)
    str = re.sub(r'\\url{([^}]*)\\slash{}',r'\\url{\1/',str)
    str = str.replace("#10^00#",'10\\textsuperscript{00}')
    str = str.replace(" #LEBENSLAUF#",'~\\pageref{lebenslauf}')
    return str
    str2 = ''
    for c in str:
        if ord(c)>127:
            str2 += '\mbox{%s}'%c
        else:
            str2 += c
    return str2

# HTML encode
def he(str):
    str = cgi.escape(str,True)
    str = str.replace("&lt;u&gt;","<u>")
    str = str.replace("&lt;/u&gt;","</u>")
    return str

# TEXT encode
def te(str):
    str = str.replace("<u>","*")
    str = str.replace("</u>","*")
    return str

# HTML Form encode
def hfe(str):
    if type(str) == bool:
        if str:
            str = "Ja"
        else:
            str = "Nein"
    str = cgi.escape(str,True)
    str = str.replace("&lt;u&gt;","*")
    str = str.replace("&lt;/u&gt;","*")
    return str

# Groups of three
def numform(num):
    rest = num
    out = ""
    while len(rest)>3:
        out += rest[0:3] + " "
        rest = rest[3:]
    out += rest
    return out
def numgeneration(num):
    return ['0','I','II','III','IV','V','VI','VII','VIII','IX','X','XI','XII','XIII','XIV'][len(num)]
def lenumform(num):
    return numform(num).replace(" ","~")

def groupnums(nums):
    grps = []
    i = 0
    while (i<len(nums)):
        count = 0
        while (i + count + 1 < len(nums)
                and nums[i+count][:-1] == nums[i+count+1][:-1]
                and cchars.index((nums[i+count][-1])) + 1 == cchars.index((nums[i+count+1][-1]))):
            count += 1
        if count > 1:
            grps.append(
                (lenumform(nums[i]) + '"~' + nums[i+count][-1]
                ,nums[i])
                )
        else:
            for j in range(0,count+1):
                grps.append(
                    (lenumform(nums[i+j])
                    ,nums[i+j])
                )
        i += count + 1
    return grps





class PersonFormatter(dict):

    def main_second_name(self):
        primary_name = self.person.get_primary_name()
        return  primary_name.get_surname()

    def full_second_name(self):
        primary_name = self.person.get_primary_name()
        prefix = primary_name.get_surname_prefix()
        if prefix: prefix += " "
        names = [ prefix + primary_name.get_surname() ]

        for name in self.person.get_alternate_names():
            type = name.get_type()
            if   type == gen.lib.NameType.BIRTH:
                why = " geb. "
            elif type == gen.lib.NameType.MARRIED:
                why = " verh. "
            elif type == gen.lib.NameType.AKA:
                why = " auch "
            elif type == u"Name nach der Hochzeit (gesch.)":
                why = " gesch. " 
            elif type == u"Name nach der Hochzeit (verw.)":
                why = " verw. " 
            elif type == u"Name nach der Adoption":
                why = " adopt. " 
            elif type == u"Namensänderung":
                why = " Namensänderung: " 
            elif type == u"Genannt":
                why = " gen. " 
            else:
                print "Unkown name type: %s for %s %s" % (type, self.main_first_name(), self.main_second_name())
                why = ", auch: " 

            prefix = name.get_surname_prefix()
            if prefix: prefix += " "
            names.append(why + prefix + name.get_surname()) 

        return "".join(names)

    def child_second_name(self):
        name_objs = []
        name_objs.extend(self.person.get_alternate_names())
        name_objs.append(self.person.get_primary_name()) # primary name last
        names = []

        for name in name_objs:
            type = name.get_type()
            if   type == gen.lib.NameType.BIRTH:
                assert len(names) == 0, "More than one birth names: "+str(names) + " for "+self.full_first_name()
                names = [ name.get_surname() ]

        for name in name_objs:
            type = name.get_type()
            if   type == gen.lib.NameType.BIRTH:
                continue
            elif type == gen.lib.NameType.MARRIED:
                why = " verh. "
            elif type == gen.lib.NameType.AKA:
                why = " auch "
            elif type == u"Name nach der Hochzeit (gesch.)":
                # no gesch. in child second name
                continue
            elif type == u"Name nach der Hochzeit (verw.)":
                why = " verw. " 
            elif type == u"Name nach der Adoption":
                why = " adopt. " 
            elif type == u"Namensänderung":
                why = " Namensänderung: " 
            elif type == u"Genannt":
                why = " gen. " 
            elif name == name_objs[-1]:
                why = " danach: "
            else:
                print "Unkown name type: %s for %s %s" % (type, self.main_first_name(), self.main_second_name())
                why = " auch: " 
            prefix = name.get_surname_prefix()
            if prefix: prefix += " "
            names.append(why + prefix + name.get_surname()) 

        return "".join(names)

    def main_first_name(self):
        primary_name = self.person.get_primary_name()
        name = primary_name.get_first_name()

        call_name = primary_name.get_call_name()
        if call_name:
            return call_name
        else:
            if name:
                return name.split()[0]
            else:
                return ""

    def full_first_name(self):
        primary_name = self.person.get_primary_name()
        name = primary_name.get_first_name()

        call_name = primary_name.get_call_name()
        if call_name:
            if call_name in name:
                name = name.replace(call_name,u"<u>%s</u>"% call_name)
            else:
                name += u"ERROR: CALLNAME"

        for attrib in self.person.attribute_list:
            if attrib.type == gen.lib.AttributeType.NICKNAME:
                #name += ', genannt %s' % attrib.get_value()
                name += ' (%s)' % attrib.get_value()

        return name

    def __init__(self,database,person):
        assert isinstance(person,gen.lib.Person), "No person passed to PersonFormatter"
        self.database = database
        self.person = person

        primary_name = person.get_primary_name()

        self['full_second_name'] = self.full_second_name()
        self['child_second_name'] = self.child_second_name()
        self['full_first_name'] = self.full_first_name()
        self['main_second_name'] = self.main_second_name()
        self['main_first_name'] = self.main_first_name()
        self['title'] = primary_name.get_title()
        self['prefix'] = primary_name.get_surname_prefix()
        if self['prefix']:
            self['prefix'] += " "

        self['full_name'] = ""
        if self['title']:
            self['full_name'] += "%(title)s " % self
        self['full_name'] += "%(full_first_name)s " % self
        self['full_name'] += "%(full_second_name)s" % self
        self['full_name'] = self['full_name'].strip()

        self['short_name'] = ("%(main_first_name)s %(prefix)s%(main_second_name)s" % self).strip()
        self['child_name'] = ("%(full_first_name)s %(child_second_name)s" % self).strip()

        self['name'] = _nd.display(person)

        self['events'] = []
        for eventref in person.get_event_ref_list():
            event = database.get_event_from_handle(eventref.ref)
                
            if event.get_type() in (gen.lib.EventType.CHRISTEN, gen.lib.EventType.BAPTISM):
                self['events'].append(
                        ('taufe', event)
                        )
            elif event.get_type() in (gen.lib.EventType.BIRTH,):
                self['events'].append(
                        ('geb', event)
                        )
            elif event.get_type() in (gen.lib.EventType.BURIAL,):
                self['events'].append(
                        ('beerdigung', event)
                        )
            elif event.get_type() in (gen.lib.EventType.DEATH,):
                if   'gefallen' in event.get_description().lower():
                    self['events'].append(
                            ('gefallen', event)
                            )
                elif 'vermisst' in event.get_description().lower():
                    self['events'].append(
                            ('vermisst', event)
                            )
                else:
                    self['events'].append(
                            ('tod', event)
                            )

        for family_handle in person.get_family_handle_list():
            family = database.get_family_from_handle(family_handle)
            for eventref in family.get_event_ref_list():
                event = database.get_event_from_handle(eventref.ref)
                if event.get_type() in (gen.lib.EventType.MARRIAGE,):
                    self['events'].append(
                            ('heirat', event)
                            )
                elif event.get_type() in (gen.lib.EventType.DIVORCE,):
                    self['events'].append(
                            ('scheidung', event)
                            )
        sort_events(self['events'])

        self['mindj'] = False
        self['gjahr'] = None
        self['tjahr'] = None

        birth_ref = person.get_birth_ref()
        if birth_ref:
            birth = database.get_event_from_handle(birth_ref.ref)

            self['gjahr'] = birth.get_date_object().get_year()

            if self['gjahr'] >= 1990:
                self['mindj'] = True

            self['gebdat'] = dd.display(birth.get_date_object())
            self['plaingebdat'] = self['gebdat']
            place_handle = birth.get_place_handle()
            self['geborthandle'] = place_handle
            place = database.get_place_from_handle(place_handle)
            if place:
                self['gebort'] = place.get_title()
                self['plaingebort'] = self['gebort']

            for eventref in person.get_event_ref_list():
                event = database.get_event_from_handle(eventref.ref)
                if event.get_type() in (gen.lib.EventType.CHRISTEN, gen.lib.EventType.BAPTISM):
                    self['gebdat'] += ", Taufe: %s" % dd.display(event.get_date_object())

                    place_handle = event.get_place_handle()
                    place = database.get_place_from_handle(place_handle)
                    if place:
                        self['gebort'] += ", Taufe: %s" % place.get_title()


        death_ref = person.get_death_ref()
        if death_ref:
            death = database.get_event_from_handle(death_ref.ref)
            self['tjahr'] = death.get_date_object().get_year()
            self['sterbdat'] = dd.display(death.get_date_object())
            self['plainsterbdat'] = self['sterbdat']
            place_handle = death.get_place_handle()
            place = database.get_place_from_handle(place_handle)
            self['sterborthandle'] = place_handle
            if place:
                self['sterbort'] = place.get_title()
                self['plainsterbort'] = self['sterbort']

            for eventref in person.get_event_ref_list():
                event = database.get_event_from_handle(eventref.ref)
                if event.get_type() in (gen.lib.EventType.BURIAL,):
                    self['sterbdat'] += ", Beerdigung: %s" % dd.display(event.get_date_object())

                    place_handle = event.get_place_handle()
                    place = database.get_place_from_handle(place_handle)
                    if place:
                        if 'sterbort' in self:
                            self['sterbort'] += ", Beerdigung: %s" % place.get_title()
                        else:
                            self['sterbort'] = "Beerdigung: %s" % place.get_title()


        self['kurzbio'] = ""
        notelist = person.get_note_list()
        for notehandle in notelist:
            note = self.database.get_note_from_handle(notehandle)
            type = note.get_type()
            if type == u"Kurzbiographie" or type == gen.lib.NoteType.PERSON:
                self['kurzbio'] += "Kurzbiographie:\n"
                self['kurzbio'] += note.get()
                self['kurzbio'] += "\n\n"

                self['kurzbiographie'] = note.get()
            elif type == u"Quellen":
                self['kurzbio'] += "Quellen:\n"
                self['kurzbio'] += note.get()
                self['kurzbio'] += "\n\n"
                
                self['quellen'] = note.get()
            elif type == u"Werke":
                self['kurzbio'] += "Werke:\n"
                self['kurzbio'] += note.get()
                self['kurzbio'] += "\n\n"
                
                self['werke'] = note.get()
            elif type == u"Redaktionsnotiz":
                self['kurzbio'] += "Redaktionsnotiz:\n"
                self['kurzbio'] += note.get()
                self['kurzbio'] += "\n\n"
                
                self['redaktionsnotiz'] = note.get()
            elif type == u"Redaktionelle Bemerkung":
                self['kurzbio'] += "Redaktionelle Bemerkung:\n"
                self['kurzbio'] += note.get()
                self['kurzbio'] += "\n\n"
                
                self['redbem'] = note.get()
            elif type == u"Interne Notiz":
                self['kurzbio'] += "Interne Notiz:\n"
                self['kurzbio'] += note.get()
                self['kurzbio'] += "\n\n"
                
                self['intnotiz'] = note.get()
            else:
                self['kurzbio'] += "Bemerkung Typ %s:\n" % type
                self['kurzbio'] += note.get()
                self['kurzbio'] += "\n\n"

        self['tot'] = False
        if death_ref or (birth_ref and self['gjahr'] <= 1910):
            self['tot'] = True
 
class TransformDict(dict):
    def __init__(self, dict, f):
        self.dict = dict
        self.f = f

    def __getitem__(self,*args):
        return self.f(self.dict.get(*args))

    def has_key(self,*args):
        return self.dict.has_key(*args)

    def __contains__(self,*args):
        return self.dict.__contains__(*args)

# vim:sw=4:ts=4:expandtab
