#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2005  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

# $Id: DescendReport.py,v 1.39.2.7 2006/04/17 03:33:34 pez4brian Exp $

"Generate files/Name Report"

#------------------------------------------------------------------------
#
# standard python modules
#
#------------------------------------------------------------------------
import os
from gettext import gettext as _
from gtk import glade


#------------------------------------------------------------------------
#
# GRAMPS modules
#
#------------------------------------------------------------------------
from PluginUtils import register_report
from ReportBase import CATEGORY_VIEW, MODE_GUI
import ManagedWindow
from gen.lib import Person

#------------------------------------------------------------------------
#
# GTK/GNOME modules
#
#------------------------------------------------------------------------
import gtk

_BORN = _('b.')
_DIED = _('d.')

#------------------------------------------------------------------------
#
# DescendantReport
#
#------------------------------------------------------------------------
class MyNameReport(ManagedWindow.ManagedWindow):

    def __init__(self,dbstate,uistate):
        """
        This report needs the following parameters (class variables)
        that come in the options class.
        

        """
	self.title = "Vornamen-Statistik"

	ManagedWindow.ManagedWindow.__init__(self, uistate, [], self.__class__)
	
	self.database = dbstate.db

	glade_file = "%s/summary.glade" % os.path.dirname(__file__)
        topDialog = glade.XML(glade_file,"summary","gramps")
        topDialog.signal_autoconnect({
            "destroy_passed_object" : self.close,
            })

	window = topDialog.get_widget("summary")
        title = topDialog.get_widget("title")
        self.set_window(window,title,self.title)

	text = ""
	name_hash = {
	    Person.MALE: {},
	    Person.FEMALE: {}
	}
	
	cursor = self.database.get_person_cursor()
	pair = cursor.first()
	while pair:
	    (handle,data) = pair
	    person = Person()
	    person.unserialize(data)

	    names = person.get_primary_name().get_first_name()
	    gender = person.get_gender()
	    for name in names.split():
		if name in name_hash[gender]:
		    name_hash[gender][name] += 1
		else:
		    name_hash[gender][name] = 1

	    pair = cursor.next()
	cursor.close()

	
        text += "Boys\n"
        text += "====\n"


	boy_names = name_hash[Person.MALE].items()
	boy_names.sort(key=lambda (_,n):n)
	for name, freq in reversed(boy_names):
	    text += "%s (%d)\n" % (name, freq)
        text += "\n"
	
        text += "Girls\n"
        text += "=====\n"

	girl_names = name_hash[Person.FEMALE].items()
	girl_names.sort(key=lambda (_,n):n)
	for name, freq in reversed(girl_names):
	    text += "%s (%d)\n" % (name, freq)

        textwindow = topDialog.get_widget("textwindow")
        textwindow.get_buffer().set_text(text)
        self.show()

    def build_menu_names(self, obj):
        return (self.title,None)

#------------------------------------------------------------------------
#
# 
#
#------------------------------------------------------------------------
register_report(
    name = 'my_name_report',
    category = CATEGORY_VIEW,
    report_class = MyNameReport,
    options_class = None,
    modes = MODE_GUI,
    translated_name = _("My Names Report"),
    status=(_("Stable")),
    description=_("Generates a statistic of first names"),
    author_name="Joachim Breitner",
    author_email="mail@joachim-breitner.de"
    )
