# coding: utf8
"""
RiehmWebSite generator.
"""

#------------------------------------------------------------------------
#
# python modules
#
#------------------------------------------------------------------------
import os
import sys
import subprocess
import md5
import time
import locale
import shutil
import codecs
import tarfile
import datetime
import tempfile
import fnmatch
import csv
import math
from gettext import gettext as _
from cStringIO import StringIO
from pyuca import Collator

try:
    set()
except:
    from sets import Set as set

#------------------------------------------------------------------------
#
# Set up logging
#
#------------------------------------------------------------------------
import logging
log = logging.getLogger(".RiehmWebSite")

#------------------------------------------------------------------------
#
# GNOME/gtk
#
#------------------------------------------------------------------------
import gtk

#------------------------------------------------------------------------
#
# GRAMPS module
#
#------------------------------------------------------------------------
import gen.lib
import const
from Filters import GenericFilter, Rules, CustomFilters
import Sort
from ReportBase import Report, ReportUtils, MenuReportOptions, CATEGORY_WEB
from gen.plug.menu import PersonOption, NumberOption, StringOption, \
                          BooleanOption, EnumeratedListOption, FilterOption, \
                          NoteOption, MediaOption, DestinationOption
from ReportBase._FileEntry import FileEntry
from ReportBase._CommandLineReport import CommandLineReport
import Errors
import Utils
from gui.utils import ProgressMeter
import GrampsLocale
from QuestionDialog import ErrorDialog, WarningDialog
from gen.display.name import displayer as _nd
import PlaceUtils


from IsaakUtils import *

#------------------------------------------------------------------------
#
# WebReport
#
#------------------------------------------------------------------------
        
class NoDoc:
    def init(self):
        pass

class RiehmBase():
    def write_stats(self, file, graphids, birthdates):
        branches = [
                ("11",  "Röttelner", []),
                ("111", "Marie Reiff", []),
                ("113", "Balmer", []),
                ("114", "Linkenheimer", []),
                ("116", "Mosbacher", []),
                ("117", "Rüppurrer", []),
                ("12",  "Ferdinand", []),
                ("13",  "Hallenser", []),
                ("13",  "Hallenser ohne Köstlin", ["138"]),
                ("132",  "Hallenser Gottfried", []),
                ("133",  "Wolmirstedter Georg", []),
                ("134",  "Grifter Paul", []),
                ("136",  "Eisenberger Hermann", []),
                ("138", "Köstlin", []),
                ("14",  "Carl", []),
                ("16",  "Kieselbronner", []),
                ("19",  "Louischen", [])
        ]

        offen_per_branch = [0]*len(branches)
        nachkommen_per_branch = [0]*len(branches)
        tot_per_branch = [0]*len(branches)
        juengstes_per_branch = [None]*len(branches)

        anzahl = 0
        partner = 0
        schwiegereltern = 0
        tot = 0
        buch_ok = 0
        buch_nicht_ok = 0
        buch_erhalt = 0
        mindj = 0
        offen = 0
        doppelt = 0

        for person_handle, num in self.persons:
            if num in self.persons.doubles:
                doppelt += 1
            else:
                anzahl += 1
                person = self.database.get_person_from_handle(person_handle)
                data = self.get_data(person,num)

                if data['partner']:
                    for famdat in data['partner']:
                        if 'spouseobj' in famdat:
                            partner += 1
                        if 'spousefatherobj' in famdat:
                            schwiegereltern += 1
                        if 'spousemotherobj' in famdat:
                            schwiegereltern += 1

                this_branches = []
                for i, (start, _ , excludes) in enumerate(branches):
                    if (num.startswith(start) and
                            reduce(lambda b, ex: b and not num.startswith(ex), excludes, True)):
                        this_branches.append(i) 

                for i in this_branches:
                    nachkommen_per_branch[i] += 1

                birth_ref = person.get_birth_ref()
                if birth_ref:
                    birthdate = self.database.get_event_from_handle(birth_ref.ref).get_date_object()
                    for i in this_branches:
                        if juengstes_per_branch[i] is None or juengstes_per_branch[i][1] < birthdate:
                            juengstes_per_branch[i] = (data['nummer'], birthdate)

                status = ""
                if  data['tot']:  
                    tot += 1
                    status = "TOT"
                
                    for i in this_branches:
                        tot_per_branch[i] += 1
                elif data['bvoe'] is True:
                    buch_ok +=1 
                    status = "OK"
                elif data['bvoe'] is False:
                    buch_nicht_ok += 1
                    status = "NOTOK"
                elif data['mindj']:
                    mindj += 1
                    status = "MINDJ"
                else:
                    offen += 1
                    for i in this_branches:
                        offen_per_branch[i] += 1
                    status = "TODO"

                graphids.write("%s %s %s %s\n" %
                        (num,status,str(data['gjahr']),str(data['tjahr'])))
                if 'gebdat' in data:
                    birthdates.write("%s %s %s\n" %
                        (num,str(data['gebdat']).split(", ")[0],str(data['kurzname'])))

                if data['bint'] == "Ja":
                    buch_erhalt +=1 

        file.write('<h3>Statistiken</h3>')
        file.write('<ul>')
        file.write('<li>Isaak-Nachkommen in der Datenbank: %d</li>' % anzahl)
        file.write('<li>Davon verstorben: %d</li>' % tot)
        file.write('<li>Buch-Einverständnis: %d (%2d)%%</li>' % (buch_ok, (buch_ok*100)/(anzahl-tot)))
        file.write('<li>Kein Buch-Einverständnis: %d (%2d)%%</li>' % (buch_nicht_ok, (buch_nicht_ok*100)/(anzahl-tot)))
        file.write('<li>Keine Angabe, aber minderjährig: %d</li>' % mindj)
        file.write('<li>Einverständnis noch nötig: %d</li>' % offen)
        file.write('<li>Buch-Interesse: %d</li>' % buch_erhalt)
        file.write('<li>Doppelt vergebene Nummern: %d</li>' % doppelt)
        file.write('<li>Partner: %d</li>' % partner)
        file.write('<li>Schwiegereltern: %d</li>' % schwiegereltern)
        file.write('</ul>')

        file.write('<h3>Pro Zweig:</h3>')
        file.write('<ul>')
        for (prefix,bname,_), offen, nachkommen, tote, juengstes in zip(branches, offen_per_branch, nachkommen_per_branch, tot_per_branch, juengstes_per_branch):
            file.write('<li>%s [%s*]: %d Nachkommen, davon %d tot und %d ohne Einverständnis. Jüngstes: %s</li>' % (bname, prefix, nachkommen, tote, offen, juengstes[0]))
        file.write('</ul>')

    def write_index(self,intern):
        if intern:
            file = open("%s/intern/index.html"%(self.dir_name), 'w')
            graphids = open("%s/intern/graph.ids"%(self.dir_name), 'w')
            birthdates = open("%s/intern/births.dat"%(self.dir_name), 'w')
        else:
            file = open("%s/index.html"%(self.dir_name), 'w')

        file.write('''
        <html>
        <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
        <title>Isaak-Buch-Projekt: Fragebögen </title>
        <style type="text/css">
        a { text-decoration:none}
        </style>
        </head>
        <body>
        <h1>Isaak-Buch-Projekt: Fragebögen</h1>
        <h3>Allgemeines</h3>
        <ul>
        <li><a href="fragebogen-brief.pdf">Anschreiben</a></li>
        ''')
        if intern:
            file.write('''
                <li><a href="blanko-intern.html">Leerer Fragebogen</a> (<a href="fragebogen-leer.pdf">Papier-Version</a>)</li>
                <li><a href="index.pdf">Übersicht nach Generationen</a></li>
                <li><a href="index_cf.pdf">Übersicht mit Kindern vor Geschwistern</a></li>
                <li><a href="data.pdf">Datenübersicht</a></li>
                <li><a href="data_a4.pdf">Datenübersicht (A4)</a></li>
                <li><a href="data_draft.pdf">Datenübersicht (ohne Bilder)</a></li>
                <li><a href="data_draft_a4.pdf">Datenübersicht (ohne Bilder, A4)</a></li>
                <li><a href="book.pdf">Buch-Entwurf</a></li>
                <li><a href="book_a4.pdf">Buch-Entwurf (A4)</a></li>
                <li><a href="adressen.csv">e-Mail-Adressliste als CSV-Datei</a></li>
                <li><a href="../../graph.png">Einfacher Graphischer Baum</a></li>
                </ul>
            ''')
            self.write_stats(file, graphids, birthdates)
            file.write('''
                <h3>Fragebögen</h3>
                <p>Wenn Sie Daten zu einem Isaak-Nachkömmling haben, der hier noch keinen eigenen Fragebogen hat, so verwenden Sie bitte den <a href="blanko-intern.html">Blanko-Fragebogen</a></p>
            ''')
        else: 
            file.write('''
                <li><a href="blanko.html">Leerer Fragebogen</a> (<a href="fragebogen-leer.pdf">Papier-Version</a>)</li>
                <li><a href="../graph.png">Einfacher Graphischer Baum</a></li>
                </ul>
                <h3>Fragebögen</h3>
                <p>Wenn Sie Daten zu einem Isaak-Nachkömmling haben, der hier noch keinen eigenen Fragebogen hat, so verwenden Sie bitte den <a href="blanko.html">Blanko-Fragebogen</a></p>
            ''')
        if not intern:
            file.write('''<p>Personen, die nur grau aufgelistet werden, leben noch und haben keine Zustimmung zur Online-Daten-Eingabe abgegeben.</p>''')
        file.write('<ul>')
        for person_handle, num in self.persons:
            if num in self.persons.doubles:
                also = self.persons.doubles[num]
                file.write('''
                        <li>%(nummer)s → %(see)s</li>
                        '''%{'num':num,'nummer':numform(num),'also':also,'see':numform(also)})

            else:
                person = self.database.get_person_from_handle(person_handle)
                data = self.get_data(person,num)
                name = "%(nachname)s, %(vorname)s" % data
                buch = " "
                if  data['tot']:    buch += "† "
                elif data['bvoe'] is True:  buch += "✓ "
                elif data['bvoe'] is False:  buch += "! "
                elif data['mindj'] and data['buchok']: buch += "(✓) "
                if not intern and not data['ovoe']:
                    file.write('''
                            <li>%(nummer)s: %(name)s %(buch)s</li>
                            '''%{'num':num,'nummer':numform(num),'name':he(name),
                                 'buch':he(buch)})
                else:
                    file.write('''
                            <li><a href="%(num)s.html">%(nummer)s: %(name)s</a>
                                <a href="%(num)s.pdf">(Papier-Version)</a> %(buch)s</li>
                            '''%{'num':num,'nummer':numform(num),'name':he(name),
                                 'buch':he(buch)})
        file.write('</ul></body></html>')

        file.close()
        if intern:
            graphids.close()
            birthdates.close()

    def write_csv(self, filename):
        titles = [
                    "Nummer",
                    "Nachname",
                    "Vorname",
                    "e-Mail-Adresse",
                    "Straße",
                    "PLZ",
                    "Ort",
                    "Land",
                    "Telefon",
                    "Aufnahme",
                    "Erhalt",
                    ]
        out = csv.DictWriter(open(filename, 'wb'), titles)
        firstrow = {}
        for t in titles: firstrow[t] = t
        out.writerow(firstrow)

        for person_handle, num in self.persons:
            if num not in self.persons.doubles:
                person = self.database.get_person_from_handle(person_handle)
                data = self.get_data(person,num)
                if not data['tot'] and (data['email'] or person.get_address_list()):
                    row = {"Nummer":num,
                           "Nachname": data['kurznachname'],
                           "Vorname": data['kurzvorname'],
                           "e-Mail-Adresse": data['email'],
                           "Aufnahme": data['addrla'],
                           "Erhalt": data['addrle'],
                           }

                    for address in person.get_address_list():
                            row['Straße'] = address.get_street()
                            row['PLZ'] = address.get_postal_code()
                            row['Ort'] = address.get_city()
                            row['Land'] = address.get_country()
                            row['Telefon'] = address.get_phone()
                    out.writerow(row)
                
                if data['partner']:
                    for famdat in data['partner']:
                        if 'spouseobj' in famdat and not famdat['tot']:
                            spouse = famdat['spouseobj']
                            if famdat['email'] or spouse.get_address_list():
                                row = {"Nummer":num,
                                       "Nachname": famdat['kurznachname'],
                                       "Vorname": famdat['kurzvorname'],
                                       "e-Mail-Adresse": famdat['email'],
                                       "Aufnahme": data['addrla'],
                                       "Erhalt": data['addrle'],
                                      }
                                for address in spouse.get_address_list():
                                        row['Straße'] = address.get_street()
                                        row['PLZ'] = address.get_postal_code()
                                        row['Ort'] = address.get_city()
                                        row['Land'] = address.get_country()
                                        row['Telefon'] = address.get_phone()
                                out.writerow(row)

    def write_emaillist(self):
        list = []

        for person_handle, num in self.persons:
            if num not in self.persons.doubles:
                person = self.database.get_person_from_handle(person_handle)
                data = self.get_data(person,num)
                for url in person.get_url_list():
                    if url.type == "E-Mail":
                        list.append("%s <%s>" % (data['kurzname'],url.get_path()))
                    elif url.type == "Homepage":
                        pass
                    else:
                        print "Unknown Internet field, type %s, value %s" % \
                                    (str(url.type), url.get_path())

                if data['partner']:
                    for famdat in data['partner']:
                        if 'spouseobj' in famdat:
                            spouse = famdat['spouseobj']
                            for url in spouse.get_url_list():
                                if url.type == "E-Mail":
                                    list.append("%s <%s>" % (famdat['kurzname'],url.get_path()))
                                elif url.type == "Homepage":
                                    pass
                                else:
                                    print "Unknown Internet field, type %s, value %s" % \
                                                (str(url.type), url.get_path())
        list.sort()

        file = open("%s/intern/email-adressen.html" % self.dir_name, 'w')
        file.write('''
        <html>
        <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
        <title>Isaak-Buch-Projekt: e-Mail-Adressen</title>
        </head>
        <body>
        <h1>Isaak-Buch-Projekt: e-Mail-Adressen</h1>
        <p>
        ''')
        file.write(he(", ".join(list)))
        file.write('''</p></body></html>''')
        file.close

    def write_eventlist(self):
        events = {}

        def add_event(person, type, event, name, num):
            date = event.get_date_object()
            if date.get_year_valid() and date.get_month_valid() and date.get_day_valid():
                day = date.get_day()
                month = date.get_month()
                year = date.get_year()

                place_handle = event.get_place_handle()
                placeobj = self.database.get_place_from_handle(place_handle)

                place = "?"
                if placeobj:
                    place = placeobj.get_title()

                if type in ('heirat','scheidung'):
                    for _, familyref in self.database.find_backlink_handles(event.get_handle(), 'Family'):
                        other = None
                        family = self.database.get_family_from_handle(familyref)
                        father_handle = family.get_father_handle()
                        mother_handle = family.get_mother_handle()
                        if person.get_handle() == father_handle:
                            other = self.database.get_person_from_handle(mother_handle)
                        elif person.get_handle() == mother_handle:
                            other = self.database.get_person_from_handle(father_handle)
                        else:
                            print "No valid partner for name?"
                            continue

                        spf = PersonFormatter(self.database, other)
                        name += " und %s" % spf['full_name']
                
                events.setdefault((month,day), []).append(
                        (year, type, num, name, place)
                )

        def add_person(person,num,main_person = False):
            pf = PersonFormatter(self.database, person)
            if 'full_name' in pf:
                name = pf['full_name']
                for type, event in pf['events']:
                    if main_person or not type in ('heirat','scheidung'):
                        add_event(person, type, event, name, num)

        for person_handle, num in self.persons:
            if num not in self.persons.doubles:
                person = self.database.get_person_from_handle(person_handle)
                add_person(person,num,True)

                data = self.get_data(person, num)
                if data['partner']:
                    for famdat in data['partner']:
                           
                        #if 'name' in famdat:
                        #    names = "%s und %s" %  (data['name'], famdat['name'])
                        #    for type, event in famdat['famevents']:
                        #        add_event(type, event, names, num)
                         
                        if 'spouseobj' in famdat:
                            add_person(famdat['spouseobj'], num)
                        if 'spousefatherobj' in famdat:
                            add_person(famdat['spousefatherobj'], num)
                        if 'spousemotherobj' in famdat:
                            add_person(famdat['spousemotherobj'], num)

        if not os.path.exists("%s/intern/events" % self.dir_name):
            os.mkdir("%s/intern/events" % self.dir_name)

        this_year = datetime.date.today().year
        for (month,day), events in events.iteritems():
            events.sort(key = lambda l:l[0])
            file = open("%s/intern/events/%02d%02d" % (self.dir_name, month, day), 'w')
            file.write("Hallo Isaak-Team,\n\n")
            file.write("heute, am %2d.%2d., jähren sich folgende Ereignisse:\n\n" % (day, month)) 
            for year, type, num, name, place in events:
                desc = "passierte etwas Unerwartetes vom typ %s\n" % type
                if type == 'taufe':
                    desc = "wurde %s in %s getauft." % (name, place)
                if type == 'geb':
                    desc = "wurde %s in %s geboren." % (name, place)
                if type == 'beerdigung':
                    desc = "wurde %s in %s bestattet." % (name, place)
                if type == 'gefallen':
                    desc = "ist %s in %s gefallen." % (name, place)
                if type == 'vermisst':
                    desc = "wurde %s in %s vermisst." % (name, place)
                if type == 'tod':
                    desc = "ist %s in %s gestorben." % (name, place)
                if type == 'heirat':
                    desc = "ließen sich %s in %s trauen." % (name, place)
                if type == 'scheidung':
                    desc = "ließen sich %s in %s scheiden." % (name, place)
                file.write(te("Vor %d Jahren, im Jahre %d, %s [%s]\n" % (this_year - year, year, desc, numform(num))))
            file.write("\n\nWenn euch diese e-Mail stört oder zu oft kommt, dann sagt mir bescheid, bis dahin werdet ihr so täglich über unsere Familiengeschichte informiert.\n\nGruß,\nJoachim\n")
            file.close()

    def write_list(self):
        for suffix, desc, list in [
                ('',   'Nach Generationen geordnet',self.persons),
                ('_cf','Kinder vor Geschwistern',self.persons.child_first()),
            ]:
            filename = "index%s.tex" % suffix
            file = open("%s/intern/%s"%(self.dir_name,filename), 'w')
            self.tex_files.append(filename)

            file.write('''
            \\documentclass[a4paper,DIV15,halfparskip,9pt,twocolumn]{scrbook}
            \\usepackage{isaak-pkgs}
            \\usepackage{scrpage2}
            \\pagestyle{scrheadings}
            \\ifoot{Nachkommensübersicht}
            \\cfoot{\\pagemark}
            \\ofoot{Daten: %(today)s}
            \\begin{document}
            \\twocolumn[{
                \\begin{center}
                    \\usekomafont{title}
                    \\Huge{Übersicht der Nachkommen Isaaks}\\\\
                    \\large{(%(desc)s)}
                \\end{center}
            }]
            \\begin{supertabular*}{\\linewidth}{llp{.7\\linewidth}}
            ''' % {'today': datetime.datetime.today(), 'desc': desc})
            for person_handle, num in list:
                if num in self.persons.doubles:
                    also = self.persons.doubles[num]
                    file.write('''%(nummer)s &  & \\raggedright $\\to$ %(see)s \\tabularnewline
                        '''%{'num':num,'nummer':numform(num),'see':numform(also)})
                else:
                    person = self.database.get_person_from_handle(person_handle)
                    data = self.get_data(person,num)
                    name = "%(nachname)s, %(vorname)s" % data
                    buch = ""
                    if  data['tot']:    buch += "\\tod"
                    elif data['bvoe'] is True:  buch += '\\dsliterary'
                    elif data['bvoe'] is False:  buch += "! "
                    elif data['mindj']: buch += '\\dsliterary*'
                    file.write('''%(nummer)s & %(buch)s &\\raggedright  %(name)s \\tabularnewline
                        '''%{'num':num,'buch':buch, 'nummer':numform(num),'name':le(name)})
            file.write('''
            \\end{supertabular*}
            \\end{document}''')

    def print_events(self, file, num, events):
        if not events:
            return

        
        first = True
        for (type, event) in events:
            dat = dd.display(event.get_date_object())
            place_handle = event.get_place_handle()
            place = self.database.get_place_from_handle(place_handle)
            ort = ''
            if place:
                ort = place.get_title()
            
            if not dat and not place:
                continue
            
            if first:
                file.write('''\\resetebenda\\begin{eventtable}''')

            facility = ''
            for attrib in event.get_attribute_list():
                if attrib.type == u"Einrichtung":
                    facility = attrib.value + ", "
            
            first = False
            file.write('''\\item[\\rlap{\\%s}\\hspace{2.5ex}~\\hfill{}%s] ''' % (type, le(dat)))
            if place:
                file.write('''%s\\ebenda{%s} ''' % (le(facility), le(ort)))
            desc = event.get_description()
            if desc and desc not in ('gefallen','vermisst'):
                file.write('''(%s)''' % le(event.get_description()))
            #if event.get_note_list():
            #    file.write('''\\newline Achtung: Weitere Notizen vorhanden''')

        if not first:
            file.write('''\\end{eventtable}%\n''')

        for (type, event) in events:
            place_handle = event.get_place_handle()
            place = self.database.get_place_from_handle(place_handle)
            if place:
                ort = place.get_title()
                self.placeindex.setdefault(ort, set()).add(num)

    def write_media_refs(self, file, person, draft, intern):
        for media_ref in person.get_media_list():
            media = self.database.get_object_from_handle(media_ref.ref)
            if not media: continue

            # Select interesting pictures
            if media.get_mime_type() in ("image/jpeg", "image/png"):
                selected = True # Standard: aufnehmen
                for attrib in media.get_attribute_list():
                    if   attrib.type == u"Isaak-Buch":
                        if attrib.value == "Ja":
                            selected = True
                        elif attrib.value == "Nein":
                            selected = False
                if not selected:
                    continue
            else:
                continue

            file.write(''' \\abbildung{\\ref{fig:%s}}''' % (media_ref.ref))

    def write_media_data(self, file, person, draft, intern):
        for media_ref in person.get_media_list():
            media = self.database.get_object_from_handle(media_ref.ref)
            if not media: continue

            twocolumn = False

            # Select interesting pictures
            if media.get_mime_type() in ("image/jpeg", "image/png"):
                selected = True # Standard: aufnehmen
                for attrib in media.get_attribute_list():
                    if   attrib.type == u"Isaak-Buch":
                        if attrib.value == "Ja":
                            selected = True
                        elif attrib.value == "Nein":
                            selected = False
                    if   attrib.type == u"Spalten":
                        if attrib.value == "2":
                            twocolumn = True
                if not selected:
                    continue
            else:
                continue

            # Pick right spot for image
            here = True
            # Is the image already somewhere?
            if media_ref.ref in self.media_added:
                here  = False
            # Is there a main person for this image? If so, only for this person    
            if here:
                for _,person_ref in self.database.find_backlink_handles(media.get_handle()):
                    other_person = self.database.get_person_from_handle(person_ref)
                    for other_media_ref in other_person.get_media_list():
                        if other_media_ref.ref == media_ref.ref:
                            for attrib in other_media_ref.get_attribute_list():
                                if attrib.type == u"Hauptperson":
                                    assert attrib.value == "Ja"
                                    if other_person.get_handle() != person.get_handle():
                                        here = False
            if not here:
                continue

            self.media_added.add(media_ref.ref)

            path = Utils.media_path_full(self.database, media.get_path())
            desc = media.get_description()

            text = ""
            notelist = media.get_note_list()
            for notehandle in notelist:
                note = self.database.get_note_from_handle(notehandle)
                text += le(note.get())

            if intern:
                text += "{\\small\\textit{Bild-ID: %s}}" % (media.get_gramps_id())

            draft_opt = ""
            if draft:
                draft_opt = "draft,"

            tcstar = ''
            if twocolumn:
                tcstar = '*'

            file.write('''%%
            \\begin{figure%s}[tb!p]
            \\raggedright%%
            \\setlength\\fboxrule{.25pt}
            \\setlength\\fboxsep{.25pt}
            \\color{rulecolor}
            \\fbox{%%
            \\setlength{\\imgwidth}{\\linewidth}%%
            \\addtolength{\\imgwidth}{-1pt}%%
            \\includegraphics[%swidth=\\imgwidth,height=1.5\\imgwidth,keepaspectratio]{%s}%%
            }%%
            \\color{textcolor}%%
            \\caption[%s]{%s %s}%%
            \\label{fig:%s}%%
            \\end{figure%s}%%
            ''' % (tcstar, draft_opt, path, le(desc), le(desc), text, media.get_handle(), tcstar))


    def write_name_index(self, file, num, person):
        names = [ person.get_primary_name() ] + person.get_alternate_names()
        for name in names:
            surname = name.get_surname()
            prefix = name.get_surname_prefix()
            if prefix:
                surname = "%s, %s" % (surname, prefix)
            first_name = name.get_first_name()
            call_name = name.get_call_name()
            if call_name:
                first_name = call_name
            elif first_name:
                    first_name = first_name.split()[0]

            for attrib in person.attribute_list:
                if attrib.type == gen.lib.AttributeType.NICKNAME:
                    first_name += ' (%s)' % attrib.get_value()

            self.nameindex.setdefault(surname,{}).setdefault(first_name, set()).add(num)

    def matches_selection(self, num):
        if self.selection:
            for sel in self.selection.split():
                if sel[0] == '-':
                    if fnmatch.fnmatch(num, sel[1:]):
                        return False
                if fnmatch.fnmatch(num, sel):
                    return True
            return False
        else:
            return True

    def  print_bios(self, file, intern, esc_data):
        if 'kurzbiographie' in esc_data:
            file.write('''\n\n\\kurzbiostart %(kurzbiographie)s '''% esc_data)
        if 'quellen' in esc_data:
            file.write('''\n\n\\quellenstart %(quellen)s '''% esc_data)
        if 'werke' in esc_data:
            file.write('''\n\n\\werkestart %(werke)s '''% esc_data)
        if 'redbem' in esc_data:
            file.write('''\n\n\\redbemstart %(redbem)s '''% esc_data)
        if intern and 'redaktionsnotiz' in esc_data:
            file.write('''\n\n\\textbf{Interne(?) Notiz:} %(redaktionsnotiz)s '''% esc_data)
        if intern and 'intnotiz' in esc_data:
            file.write('''\n\n\\textbf{Interne Notiz:} %(intnotiz)s '''% esc_data)
        # This is for some reason required to get rid of an extra empty line at the end
        # of the column. \hrule worked, \par etc. not. Probably because \hrule leaves
        # the horizontal mode
        # file.write('\\hrule height0pt')

    def write_book_pdf(self, target_file, intern, draft):
        tmpdir = tempfile.mkdtemp('', 'riehm-extract-')
        try:
            self.persons = Descendants(self.database,self.start_person)    

            self.progress.set_pass(_('Writing maps'),2)
            self.write_map(tmpdir+"/deutschland.pdf","deutschland")
            self.progress.step()
            self.write_map(tmpdir+"/europa.pdf","europa")
            self.progress.step()
            self.write_map(tmpdir+"/welt.pdf","welt")
            self.progress.step()

            (basename,ext) = os.path.splitext(os.path.basename(target_file))
            target_file_a4 = os.path.dirname(target_file) + "/" + basename + "_a4.pdf"
            assert ext == '.pdf'

            # writing book
            self.write_book(tmpdir + "/" + basename + ".tex", intern, draft)

            # building book
            texenv = { 'TEXINPUTS': '.:' + self.source_path + ':',
                       'MFINPUTS':  self.source_path + ':',
                       'AFMFONTS':  self.source_path + ':',
                       'TTFONTS':  self.source_path + ':',
                       'TFMFONTS':  self.source_path + ':',
                       'ENCFONTS':  self.source_path + ':',
                       'HOME':      tmpdir,
                     }

            if intern:
                if draft:
                    self.progress.set_pass(_('Building Book (intern,draft)'),4)
                else:
                    self.progress.set_pass(_('Building Book (intern)'),4)
            else:
                if draft:
                    self.progress.set_pass(_('Building Book (draft)'),4)
                else:
                    self.progress.set_pass(_('Building Book'),4)

            subprocess.call(["pdflatex","-halt-on-error","-interaction","errorstopmode",basename + ".tex"], cwd=tmpdir, env=texenv)
            self.progress.step()
            subprocess.call(["pdflatex","-halt-on-error","-interaction","errorstopmode",basename + ".tex"], cwd=tmpdir, env=texenv)
            self.progress.step()
            subprocess.call(["pdflatex","-halt-on-error","-interaction","errorstopmode",basename + ".tex"], cwd=tmpdir, env=texenv)
            self.progress.step()

            tmp_pdf_file = tmpdir + "/" + basename + ".pdf"
            if os.path.exists(tmp_pdf_file):
                subprocess.call(["pdfjam","--noautoscale","true","--frame","true", tmp_pdf_file, "-o", target_file_a4], cwd=tmpdir)
                self.progress.step()
                shutil.move(tmp_pdf_file, target_file)
                #shutil.move(tmpdir + "/" + basename + ".tex", os.path.dirname(target_file))
            else:
                print "No pdf file generated!"
        finally:
            shutil.rmtree(tmpdir)
            pass

    def write_map(self,target_file,what):

        birthplacemap = {}

        for person_handle, num in self.persons:
            if num in self.persons.doubles:
                pass
            # Always complete map
            #elif not self.matches_selection(num):
            #    pass
            else:
                person = self.database.get_person_from_handle(person_handle)
                birth_ref = person.get_birth_ref()
                if birth_ref:
                    birth = self.database.get_event_from_handle(birth_ref.ref)
                    place_handle = birth.get_place_handle()
                    if place_handle:
                        if place_handle not in birthplacemap:
                            birthplacemap[place_handle] = 0
                        birthplacemap[place_handle] += 1
        birthplacelist = birthplacemap.items()
        birthplacelist.sort(key=(lambda (a,b): (b,a)),reverse=True)


        (basename,ext) = os.path.splitext(os.path.basename(target_file))
        assert ext == "pdf"
        psfilename = os.path.dirname(target_file) + "/" + basename + ".ps"
        templatefilename = self.source_path + "/" + basename + ".ps"
        shutil.copy(templatefilename, psfilename)

        if what=="deutschland":
            #common_opts = ["-R5/21.5/46.8/55","-JM14ch"]
            #common_opts = ["-R5/16/46.8/55","-JM14c","-P"]
            common_opts = ["-R5/15.5/46.8/55","-JB10/50/45/55/14c","-P"]
        elif what=="osteuropa":
            common_opts = ["-R14/23/49/55","-JM14ch"]
        elif what=="europa":
            common_opts = ["-R-12/24/46/60","-JB6/50/45/55/14c","-P"]
        elif what=="welt":
            common_opts = ["-R-130/150/-60/75","-JN14c","-P","--ELLIPSOID=Sphere"]

        common_opts += ["-V","-K","--HISTORY=FALSE","-O"]

        psfile = file(psfilename,'a')

        if what == "welt":
            cmd = subprocess.Popen(["GMT","psxy"]+ common_opts +[
                "-L",
                "-W0.8p,50",
                ],
                stdin=subprocess.PIPE,
                stdout=psfile)
            for coords in [(-12,46), (-12, 60), (24,60), (24,46)]:
                cmd.stdin.write("%s %s\n" % coords)
            cmd.stdin.close()
            cmd.wait()

        if what in ("europa"): #("europa","welt"):
            cmd = subprocess.Popen(["GMT","psxy"]+ common_opts +[
                "-L",
                "-W0.8p,50",
                ],
                stdin=subprocess.PIPE,
                stdout=psfile)
            for coords in [(5,46.8), (5, 55), (15.5,55), (15.5,46.8)]:
                cmd.stdin.write("%s %s\n" % coords)
            cmd.stdin.close()
            cmd.wait()

        cmd = subprocess.Popen(["GMT","psxy"]+ common_opts +[
            "-Scp",
            #"-Wfaint,150",
            "-G50",
            ],
            stdin=subprocess.PIPE,
            stdout=psfile)
        for place_handle, count in birthplacelist:
            place = self.database.get_place_from_handle(place_handle)
            lat   = place.get_latitude()
            lon   = place.get_longitude()
            if not (lat and lon):
                sys.stderr.write('No coordiantes for place %s\n' % place.get_title())
                continue

            lat, lon = PlaceUtils.conv_lat_lon(lat,lon,"D.D8")
            if not (lon and lat):
                sys.stderr.write('Could not convert coordiantes for place %s\n' % place.get_title())
                continue

            #if what=="welt":
            #    if -12 <= float(lon) <= 37 and 36 <= float(lat) <= 60:
            #        continue
            #
            #if what=="europa":
            #    if 5 <= float(lon) <= 15.5 and 46.8 <= float(lat) <= 55:
            #        continue

            size = math.sqrt(count) * 2
            cmd.stdin.write("%s %s %.2f\n" % (lon,lat,size))
        cmd.stdin.close()
        cmd.wait()

        common_opts.remove("-K")
        subprocess.call(["GMT","psbasemap"]+common_opts+[
            "-Bp",
            "--BASEMAP_TYPE=plain",
            ],
            stdout=psfile)

        psfile.close()

        subprocess.call(["GMT","ps2raster",psfilename,"-Tf","-V","-A"])


    def write_kml(self,target_file,intern):

        self.persons = Descendants(self.database,self.start_person)    

        places = set()
        for person_handle, num in self.persons:
            if num in self.persons.doubles:
                pass
            elif not self.matches_selection(num):
                pass
            else:
                person = self.database.get_person_from_handle(person_handle)
                data = self.get_data(person, num)
                for (type, event) in data['events']: 
                    place_handle = event.get_place_handle()
                    if place_handle: places.add(place_handle)
                if data['partner']:
                    for famdat in data['partner']:
                        for (type, event) in famdat['events']: 
                            place_handle = event.get_place_handle()
                            if place_handle: places.add(place_handle)
                    for k in ['vatergeborthandle','muttergeborthandle','vatersterborthandle', 'muttersterborthandle']:
                        if k in famdat:
                            if famdat[k]: places.add(famdat[k])

        out = file(target_file, 'w')
        out.write('''<?xml version="1.0" encoding="UTF-8"?>
            <kml xmlns="http://www.opengis.net/kml/2.2">
            <Document>''')

        c = Collator(os.path.dirname(__file__) + "/allkeys.txt")

        places = list(places)
        places.sort(key=lambda h: c.sort_key(self.database.get_place_from_handle(h).get_title()))

        for place_handle in places:
            place = self.database.get_place_from_handle(place_handle)
            lat   = place.get_latitude()
            lon   = place.get_longitude()
            if not (lat and lon):
                sys.stderr.write('%s\n' % place.get_title())
                continue
            
            lat, lon = PlaceUtils.conv_lat_lon(lat,lon,"D.D8")
            if not (lon and lat):
                sys.stderr.write('Could not convert coordiantes for place %s\n' % place.get_title())
                continue

            out.write('''<Placemark>
                <name>%s</name>
                <Point>
                  <coordinates>%s,%s</coordinates>
                </Point>
              </Placemark>''' % (place.get_title(), lon, lat))
        out.write('''</Document></kml>''')
        out.close()


    def write_book(self,filename,intern,draft):
        self.progress.set_pass(_('Writing book'),len(self.persons))
        self.media_added = set()

        self.nameindex = {}
        self.placeindex = {}
        self.partner_seen_at = {}

        if intern:
            confidentially = "internen"
        else:
            confidentially = "familiären"

        if self.selection and self.selection != "1*":
            auszug = '''Dies ist ein Auszug der vorhandenen Daten (Filter: %s)\\\\''' % le(self.selection)
            internetornot = '\\Internettrue'
        else:
            auszug = ''
            internetornot = '\\Internetfalse'
        
        file = open(filename, 'w')
        file.write('''
        \\documentclass[twoside,a4paper,DIV15,parskip=off,9pt]{scrbook}

        \\newif\\ifInternet
        %(internetornot)s

        \\newcommand{\\timestamp}{%(today)s}
        \\newlength{\\imgwidth}
        \\usepackage{isaak-pkgs}
        \\usepackage{isaak-booksize}
        \\usepackage{numind}
        \\usepackage{fix2col}
        \\usepackage{ebenda}
        %%\\usepackage{lifeline}
        \\usepackage{lifesepline}

        \\title{Daten der Nachkommen Isaaks}
        \\subtitle{%(auszug)sNur für den %(confidentially)s Gebrauch!}
        \\author{Isaak-Buch-Team}

        \\color{textcolor}
        \\begin{document}
        \\input{schmutztitel}

        \\twocolumn
        \\chapter*{Inhaltsverzeichnis}
        \\tableofcontents
        \\chapter*{Abbildungen}
        \\listoffigures
        \\clearpage
    
        %% Add lifeline, abusing ifoot
        %%\\ifoot{\\rightmark\\drawlifelineimage{}}

        \\onecolumn
        \\input{vorwort}

        \\input{lebenslauf}

        \\input{vorfahren}

        \\addtocontents{toc}{\\protect\\pagebreak}
        \\twocolumn
        \\addchap{Isaaks und Charlottes Nachkommen}
        \\begin{hauptteil}
        %%\\raggedright
        ''' % {'today': datetime.datetime.today().strftime("%d.%m.%Y %H:%M"),
               'confidentially' : confidentially,
               'auszug' : auszug,
               'internetornot' : internetornot,
               })

        def f(s):
            if s is None:
                return ''
            else:
                return le(s)

        for person_handle, num in self.persons:
            self.progress.step()
            if num in self.persons.doubles:
                pass
            elif not self.matches_selection(num):
                pass
            else:
                person = self.database.get_person_from_handle(person_handle)
                data = self.get_data(person, num)
                esc_data = TransformDict(data,f)
                
                if not intern and not data['buchok']:
                    file.write('\\lifeseplineunknown{%(nummer)s}' % data)
                    file.write('''
                    \\addcontentsline{toc}{subsection}{%(nummer)s}%%
                    \\label{%(num)s}%%
                    \\markboth{{%(generation)s}{%(nummer)s}}{{%(generation)s}{%(nummer)s}}%%
                    \\personheader{%(nummer)s}''' % esc_data)
                    file.write('''%
                            \\unskip%
                            \\par%
                            %\\pagebreak[2]%
                            \\addvspace{8mm plus 0cm minus 4mm}%
                        ''')
                    continue

                
                if data['gjahr']:
                    if  data['tjahr']:
                        file.write('\\lifeseplinetot{%(nummer)s}{%(gjahr)d}{%(tjahr)d}' % data)
                    else:
                        if data['tot']:
                            year = min(2010, data['gjahr'] + 100)
                            file.write('\\lifeseplinemaybetot{%(nummer)s}{%(gjahr)d}' % data)
                            file.write('{%d}' % year)
                        else:
                            file.write('\\lifesepline{%(nummer)s}{%(gjahr)d}' % data)
                else:
                    if data['tjahr']:
                            year = max(1799, data['tjahr'] - 100)
                            file.write('\\lifeseplinetotnobirth{%(nummer)s}' % data)
                            file.write('{%d}' % year)
                            file.write('{%(tjahr)d}' % data)
                    else:
                        file.write('\\lifeseplineunknown{%(nummer)s}' % data)
                
                file.write('''%%
                \\addcontentsline{toc}{subsection}{%(nummer)s -- %(kurzname)s}%%
                \\label{%(num)s}%%
                \\markboth{{%(generation)s}{%(nummer)s}}{{%(generation)s}{%(nummer)s}}%%
                \\personheader{\\putright{%(nummer)s – %(kurzname)s}{''' % esc_data)

                # Alleinerziehend?
                if  data['partner'] and data['partner'][0]['geschl'] == gen.lib.Person.UNKNOWN:
                    file.write(self.family_icons(data,single=True))
                else:
                    file.write(self.family_icons(data))
                file.write('''}}\\par\\addvspace{5.3mm}\n''')
                
                self.write_name_index(file, num, person)

                if data['obj'].get_gramps_id() in self.partner_seen_at:
                    file.write('''Siehe %s, \\numberreference{[%s]}.''' %
                            self.partner_seen_at[data['obj'].get_gramps_id()])
                else: 
                    if intern and not data['buchok']:
                        file.write('\n\n\\textbf{Diese Person wird im Buch nicht erscheinen, da keine Zustimmung zur Veröffentlichung vorliegt!}\n\n')


                    #file.write('''\\putright{''')
                    if data['name'] != data['kurzname']:
                        file.write('''{%%
                                \\setlength{\\parfillskip}{0pt plus 1fill}%%
                                \\RaggedRight%%
                                %(name)s%%
                                \\par}'''% esc_data)
                    #file.write('}{')
                    #self.write_media_refs(file, person, draft, intern)
                    #file.write('}')
                    self.write_media_data(file, person, draft, intern)

                    #if data['gjahr']:
                    #    if  data['tjahr']:
                    #        file.write('\\lifelinetot{%(nummer)s}{%(gjahr)d}{%(tjahr)d}' % data)
                    #    else:
                    #        file.write('\\lifeline{%(nummer)s}{%(gjahr)d}' % data)
                    

                    self.print_events(file, num, data['events'])
                    self.print_bios(file, intern, esc_data)

                    if intern:
                        if data['adresse'] or data['email']:
                            file.write('\n\n{\\sectfont Adressdaten:} (erscheinen nicht im Buch)\\\\\n')
                            file.write(esc_data['adresse'])
                            file.write("\n\n")
                            file.write("\\url{%s}" % data['email'])

                    if data['partner']:
                        for famdat in data['partner']:
                            famdat_esc = TransformDict(famdat,f)

                            if famdat['geschl'] != gen.lib.Person.UNKNOWN:
                                file.write('''%
                                    \\unskip%
                                    \\par%
                                    %\\pagebreak[2]%
                                    \\addvspace{5mm plus 0cm minus 2mm}%
                                ''')
                                if 'gjahr' in famdat and famdat['gjahr']:
                                    if 'tjahr' in famdat and famdat['tjahr']:
                                        file.write('\\lifeseplinetot{%(nummer)s}{%(gjahr)d}{%(tjahr)d}' % famdat)
                                    else:
                                        if famdat['tot']:
                                            year = min(2010, famdat['gjahr'] + 100)
                                            file.write('\\lifeseplinemaybetot{%(nummer)s}{%(gjahr)d}' % famdat)
                                            file.write('{%d}' % year)
                                        else:
                                            file.write('\\lifesepline{%(nummer)s}{%(gjahr)d}' % famdat)
                                else:
                                    if famdat['tjahr']:
                                            year = max(1799, famdat['tjahr'] - 100)
                                            file.write('\\lifeseplinetotnobirth{%(nummer)s}' % famdat)
                                            file.write('{%d}' % year)
                                            file.write('{%(tjahr)d}' % famdat)
                                    else:
                                        file.write('\\lifeseplineunknown{%(nummer)s}' % famdat)

                                file.write('''\\spouseheader{\\putright{%(kurzname)s}{'''% famdat_esc)
                                file.write(self.family_icons(data,active_famdat=famdat))
                                file.write('''}}''')

                                file.write('''\\vspace{3.6mm}\n''')

                                self.partner_seen_at[famdat['spouseobj'].get_gramps_id()] = (esc_data['kurzname'],data['nummer'])
                                
                                if famdat['name'] != famdat['kurzname']:
                                    file.write('''{%%
                                            \\setlength{\\parfillskip}{0pt plus 1fill}%%
                                            \\RaggedRight%%
                                            %(name)s%%
                                            \\par}'''% famdat_esc)

                                if 'spouseobj' in famdat:
                                    self.write_media_data(file, famdat['spouseobj'], draft, intern)
                                    self.write_name_index(file, num, famdat['spouseobj'])
                                if 'spousefatherobj' in famdat:
                                    self.write_name_index(file, num, famdat['spousefatherobj'])
                                if 'spousemotherobj' in famdat:
                                    self.write_name_index(file, num, famdat['spousemotherobj'])

                                self.print_events(file, num, famdat['events'])

                                if 'vatername' not in famdat and 'muttername' not in famdat:
                                    pass
                                else:
                                    file.write('''\\begin{parenttable}''')
                                    if 'vatername' in famdat:
                                        file.write('''\\fatheritem %(vatername)s''' % famdat_esc)
                                        if 'vaterplaingebort' in famdat or 'vaterplaingebdat' in famdat:
                                            file.write(''', \\geb~''')
                                            if 'vaterplaingebdat' in famdat:
                                                file.write('''%(vaterplaingebdat)s ''' % famdat_esc)
                                            if 'vaterplaingebort' in famdat:
                                                file.write('''\\ebenda{%(vaterplaingebort)s}''' % famdat_esc)
                                                self.placeindex.setdefault(famdat['vaterplaingebort'], set()).add(data['num'])
                                    if 'muttername' in famdat:
                                        file.write('''\\motheritem %(muttername)s''' % famdat_esc)
                                        if 'mutterplaingebort' in famdat or 'mutterplaingebdat' in famdat:
                                            file.write(''', \\geb~''')
                                            if 'mutterplaingebdat' in famdat:
                                                file.write('''%(mutterplaingebdat)s ''' % famdat_esc)
                                            if 'mutterplaingebort' in famdat:
                                                file.write('''\\ebenda{%(mutterplaingebort)s}''' % famdat_esc)
                                                self.placeindex.setdefault(famdat['mutterplaingebort'], set()).add(data['num'])
                                    file.write('''\\end{parenttable}''')

                                self.print_bios(file, intern, famdat_esc)

                            if intern:
                                if famdat['adresse'] or famdat['email']:
                                    file.write('\n\n{\\sectfont Adressdaten:} (erscheinen nicht im Buch)\\\\\n')
                                    file.write(famdat_esc['adresse'])
                                    file.write("\n\n")
                                    file.write("\\url{%s}" % famdat['email'])

                            # Kinder pro Partner:
                            # if data['kinder']:
                            if famdat['kinder']:
                                file.write('''
                                
                                %%\\pagebreak[1]%%
                                \\addvspace{5mm plus 1mm minus 2mm}%%
                                \\resetebenda%%
                                %\\childrenheader
                                \\begin{kinderliste}

                                    ''')
                                for kinddat in famdat['kinder']:
                                    file.write('''
                                            \\item[\\numberreference{%(nummer)s}\\hfill] %%
                                        ''' % TransformDict(kinddat,f))
                                    if ((not intern and not kinddat['buchok']) or 
                                         not self.matches_selection(kinddat['num'])):
                                        
                                        if kinddat['geschl'] == gen.lib.Person.MALE:
                                            file.write('''Ein Sohn''')
                                        elif kinddat['geschl'] == gen.lib.Person.FEMALE:
                                            file.write('''Eine Tochter''')
                                        else:
                                            file.write('''Ein Kind''')
                                    else:
                                        file.write('''\\hyperref[%(num)s]{%(kindname)s}%%
                                            ''' % TransformDict(kinddat,f))
                                        if 'gebdat' in kinddat:
                                            file.write('''\\nopagebreak\\\\\\geb{}~%(plaingebdat)s \\ebenda{%(plaingebort)s}''' % TransformDict(kinddat,f))
                                file.write('''\end{kinderliste}''')

                    # alle Kinder am Stück:
                    # if data['kinder']:
                    if False:
                        file.write('''
                        
                        {\\sectfont Kinder:}
                                \\begin{compactitem}
                            ''')
                        for kinddat in data['kinder']:
                            file.write('''
                            \\item \\hyperref[%(num)s]{%(nummer)s}  %(kindname)s
                                ''' % TransformDict(kinddat,f))
                            if 'gebdat' in kinddat:
                                file.write('''\\\\ \\geb{}~%(gebdat)s''' % TransformDict(kinddat,f))
                        file.write('''\end{compactitem}''')

                file.write('''%
                        \\unskip%
                        \\par%
                        %\\pagebreak[2]%
                        \\addvspace{8mm plus 0cm minus 4mm}%
                    ''')

        file.write('''
        %\\clearpage
        % Disabling lifeline
        %\\ifoot{\\rightmark}
        \\end{hauptteil}
        ''')



        file.write('''
        \\onecolumn
        \\addchap{Alphabetisches Personenverzeichnis}
        
        \\lettrine[lines=3]{D}{}as Personenverzeichnis enthält alle \\ifInternet hier verzeichneten \\else direkten\\fi Nachkommen von Isaak und
        Charlotte, deren eventuelle Partner oder Partnerinnen sowie deren
        Eltern, soweit bekannt. Zu jeder Person wird der Familienname und der
        Rufname, gegebenenfalls mit Spitznamen,  sowie die jeweilige Abstammungsnummer (zur Erläuterung der
        Systematik der Abstammungsnummern vgl. Seite~\\pageref{nummernsystem}) angegeben. Über
        die Abstammungsnummer kann die betreffende Person dann im Buch gefunden
        werden. Da die Partner und Partnerinnen und deren Eltern keine eigene
        Abstammungsnummer besitzen, wird bei ihnen auf die Nummer des
        Isaak-Nachkommen verwiesen. 

        In diesem Verzeichnis sind \\textit{nicht} enthalten alle Personen, die
        in den Kurzbiografien erwähnt wurden, aber selbst nicht zur direkten
        Familie gehören. Das Verzeichnis erschließt auch nicht die in der
        Einführung oder in der Lebensbeschreibung von Isaak erwähnten Personen.

        \\begin{multicols}{3}
        \\begin{theindex}
        \\raggedright
        ''')

        c = Collator(os.path.dirname(__file__) + "/allkeys.txt")

        def remove_title(str):
            if not str:
                return "ZZZZZZ"
            for prefix in ["von ", "van ", "de "]:
                if str.startswith(prefix):
                    str = str[len(prefix):]
            return str

        surnames = self.nameindex.keys()
        surnames.sort(key=lambda s: c.sort_key(remove_title(s)))
        for surname in surnames:
            if surname:
                file.write('''\\item {\\color{headercolor}%s}''' % le(surname))
            else:
                file.write('''\\item \\textit{Nachname unbekannt}''')
            firstnames = self.nameindex[surname].keys()
            firstnames.sort()
            for firstname in firstnames:
                if firstname:
                    file.write('''\\subitem %s''' % le(firstname))
                else:
                    file.write('''\\subitem \\textit{Vorname unbekannt}''')

                nums = list(self.nameindex[surname][firstname])
                nums.sort(cmp=cmp_num)
                for num in nums:
                    file.write(''', \\hyperref[%s]{%s}''' % (num,lenumform(num)))
        file.write('''
        \\end{theindex}
        \\end{multicols}
        ''')

        file.write('''
        \\onecolumn
        \\addchap{Personenverzeichnis nach Familienzweigen}

        \\lettrine[lines=3]{I}{}m Gegensatz zur Darstellung im Inhaltsverzeichnis und der Nachkommen von Isaak und Charlotte im Hauptteil des Buches, bei denen die Reihenfolge der Personen durch die Generationen bestimmt wird (1. Generation, vollständige 2. Generation, vollständige 3. Generation etc.), eröffnen wir mit dieser Übersicht einen alternativen Zugang auf die Nachkommen. Hier folgt die Darstellung dem Prinzip „Kinder vor Geschwistern“, bewegt sich also immer durch die Generationen abwärts, um dann wieder oben zu beginnen. Der Vorteil dieser Darstellung ist, dass die enger verwandten Familienangehörigen über die Generationen hinweg, etwa die Linkenheimer Riehms \\numberreference{[114 ff.]} oder die Köstlins \\numberreference{[138 ff.]}, alle unmittelbar aufeinander folgen.  
        

        \\begin{multicols}{3}
        \\makeatletter
        \\renewcommand*\\@idxitem{\\par\\hangindent 20\\p@}
        \\makeatother
        \\begin{theindex}
        \\resetebenda
        \\raggedright
        ''')
        for person_handle, num in self.persons.child_first():
            if num in self.persons.doubles:
                pass
            elif not self.matches_selection(num):
                pass
            else:
                person = self.database.get_person_from_handle(person_handle)
                data = self.get_data(person, num)
                esc_data = TransformDict(data,f)
                
                if not intern and not data['buchok']:
                    file.write('\\item %(nummer)s\n' % data)
                    continue

                surname = data['prefix'] + data['kurznachname']
                short_surname = re.sub(r'([A-Z])[^ .-]+', r'\1.', surname)

                file.write('''\\item %s – %s \\reprepl{{\\color{headercolor}%s}}{%s}\n''' % (esc_data['nummer'], esc_data['kurzvorname'], le(surname), le(short_surname)))

                
        file.write('''
        \\end{theindex}
        \\end{multicols}
        ''')

        file.write('''
        \\addchap{Ortsverzeichnis}

        \\lettrine[lines=3]{D}{}as folgende Ortsverzeichnis enthält alle Orte, die im Personenteil
        dieses Buches, als Geburts-, Tauf-, Heirats-, Sterbe- und Begräbnisorte
        angeben wurden. Man beachte, dass diese Angaben bei den einzelnen
        Personen nicht immer vollständig sind. Wir haben uns bemüht, als erstes
        den Ortsnamen in der Form anzugeben, wie er zum jeweiligen Zeitpunkt
        gebräuchlich war, danach – bei Namensänderungen durch Eingemeindungen
        etc. – den heutigen Namen. Außerdem haben wir bei kleinen und eher
        unbekannten Orten Ergänzungen der Region oder Verweise auf eine in der
        Nähe liegende größere Stadt vorgenommen. Bei Orten im Ausland wurde
        immer der jeweilige Staat ergänzt, bei Orten in Deutschland wurde
        darauf verzichtet. Nach den Ortsangaben folgt die Abstammungsnummer
        (zur Erläuterung der Systematik der Abstammungsnummern vgl.
        Seite~\\pageref{nummernsystem}), über die die Person gefunden werden kann,
        die einen Bezug zum jeweiligen Ort aufweist.
        Die Ordnung der Abstammungsnummern folgt der Ordnung der Personen im
        Hauptteil dieses Buches. Sind bei einem Ort mehrere Kinder ohne
        Unterbrechung genannt, werden sie zusammengenommen; zum Beispiel Basel:
        143-6 statt 143, 144, 145, 146.
        Da die Partner und
        Partnerinnen und deren Eltern keine eigene Abstammungsnummer besitzen,
        wird bei ihnen auf die Nummer des Isaak-Nachkommen verwiesen. 

        In diesem Verzeichnis sind \\textit{nicht} enthalten alle Orte, die nur
        in den Kurzbiografien erwähnt wurden, z.B. Wohnorte, soweit sie nicht
        auch Geburts-, Tauf-, Heirats-, Sterbe- oder Begräbnisorte sind. Es
        sind auch nicht die Orte nachgewiesen, die in der Einführung und in der
        Lebensbeschreibung von Isaak erwähnt werden.

        \\begin{multicols}{2}
        \\makeatletter
        \\renewcommand*\\@idxitem{\\par\\hangindent 20\\p@}
        \\makeatother
        \\begin{theindex}

        \\raggedright
        ''')

        places = self.placeindex.keys()
        places.sort(key=c.sort_key)
        for place in places:
            m = re.match("^(.*?)((?:,| bei| im| in| \(| auf).*)?$",place)
            main = m.group(1)
            rest = m.group(2) or ""
            file.write('''\\item {\\color{headercolor}%s}%s''' % (le(main), le(rest)))
            nums = list(self.placeindex[place])
            nums.sort(cmp=cmp_num)
            grps = groupnums(nums)
            for num,href in grps:
                file.write(''', \\hyperref[%s]{%s}''' % (href,num))
        file.write('''

        \\end{theindex}
        \\end{multicols}

        %%\\input{symbole}

        \\end{document}''')

    def family_icons(self, data, single=False, active_famdat=None):
        ret = "{\\fontsize{14}{14}\\selectfont"
        if   (data['geschl'] == gen.lib.Person.MALE):
            if active_famdat is None:
                ret += "\\Mansym"
            else:
                ret += "\\mansym"
        elif (data['geschl'] == gen.lib.Person.FEMALE):
            if active_famdat is None:
                ret += "\\Womansym"
            else:
                ret += "\\womansym"
        else:
            ret += "?"

        if data['partner']:
            first = True
            for famdat in data['partner']:
                active = (first and single) or famdat is active_famdat
                first = False
                if   (famdat['geschl'] == gen.lib.Person.MALE):
                    if active:
                        ret += "\\Mansym"
                    else:
                        ret += "\\mansym"
                elif (famdat['geschl'] == gen.lib.Person.FEMALE):
                    if active:
                        ret += "\\Womansym"
                    else:
                        ret += "\\womansym"
                else:
                    # Kein Partnersymbol falls alleinstehend
                    ret += ""

                if famdat['kinder']:
                    for kinddat in famdat['kinder']:
                        if   (kinddat['geschl'] == gen.lib.Person.MALE):
                            if active:
                                ret += "\\Boysym"
                            else:
                                ret += "\\boysym"
                        elif (kinddat['geschl'] == gen.lib.Person.FEMALE):
                            if active:
                                ret += "\\Girlsym"
                            else:
                                ret += "\\girlsym"
                        else:
                            ret += "?"
        ret += "}" 
        return ret


    def write_html(self, data, num, intern):
        if not intern and not data['ovoe']:
            return

        if intern:
            file = open('%s/intern/%s.html'%(self.dir_name,num), 'w')
        else:
            file = open('%s/%s.html'%(self.dir_name,num), 'w')

        def f(s):
            if s is None:
                return ''
            else:
                return hfe(s)

        esc_data = TransformDict(data,f)

        title = "%(vorname)s %(nachname)s (%(nummer)s)" % esc_data

        # HTML-Header
        file.write('''
        <html>
        <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
        <style type="text/css">
            dt  { font-weight: bold }
        </style>
        <title>Isaak-Buch-Projekt: Fragebogen zu %s</title>
        </head>
        <body>
        <h1>Isaak-Buch-Projekt: Fragebogen zu %s</h1>
        <form action="send" method="POST">
        <h2>Bemerkungen</h2>
        <p>Wenn Sie uns, neben den reinen Daten des Fragebogens, weitere Informationen mitgeben wollen, so können Sie dafür das folgende Feld benutzen. </p>
    	<textarea name="bemerkung" rows="5" cols="80"></textarea>

        <p>Bitte beachten Sie beim Ausfüllen der Felder die Hinweise im <a href="http://riehm.name/fragebogen/fragebogen-brief.pdf">Anschreiben</a>. Abweichend davon gilt, dass ggf. der Rufname nicht unterstrichen, sondern mit Sternchen markiert wird (z.B. „<tt>Heinrich *Isaak*</tt>“)</p>
        <h2>Angaben zur Person in direkter Linie zu Isaak Riehm</h2>
        <input type="hidden" name="num" value="%s"/>
        ''' % (title,title,num))
        
        def mkfield(dat, desc,val,prefix="",expl=None):
            expl_html = ""
            if expl:
                expl_html = "%s<br/>" % expl
            return '''
                <dt>%(desc)s</dt><dd>
                %(expl)s
                <input type="text" name="%(key)s" value="%(value)s" size="80"/>
                <input type="hidden" name="%(key)s_old" value="%(value)s"/>
                </dd> ''' % {'key': prefix+val, 'desc':desc, 'value':dat[val], 'expl':expl_html}
        
        def mkinfo(dat, desc,val,prefix="",expl=None):
            expl_html = ""
            if expl:
                expl_html = "%s<br/>" % expl
            return '''
                <dt>%(desc)s</dt><dd>
                %(expl)s
                %(value)s
                </dd> ''' % {'key': prefix+val, 'desc':desc, 'value':dat[val], 'expl':expl_html}
        
        def mklargefield(dat, desc,val,prefix=""):
            return '''
                <dt>%(desc)s</dt><dd>
                <textarea name="%(key)s" rows="15" cols="80">%(value)s</textarea>
                <input type="hidden" name="%(key)s_old" value="%(value)s"/>
                </dd> ''' % {'key': prefix+val, 'desc':desc, 'value':dat[val]}

        file.write('<dl>')
        for desc, val, expl in (
                            ('Nachnamen',   'nachname',
                                            'Bitte den aktuellen oder letzten Nachnamen sowie etwaige vorherige Nachnamen angeben. Z.B.: <tt>Riehm geb. Balmer</tt>'),
                            ('Vornamen',    'vorname',
                                            'Bitte alle Vornamen angeben und den Rufnamen mit * einfassen. Z.B.: <tt>*Heinrich* Karl, genannt Schmunzel-Heiner</tt>'),
                            ('Titel',       'title', ''),
                            ('Geburtsdatum','gebdat', ''),
                            ('Geburtsort',  'gebort', ''),
                            ('Sterbedatum', 'sterbdat', ''),
                            ('Sterbeort',   'sterbort', '') ):
            file.write(mkfield(esc_data,desc,val, expl=expl))
        file.write(mklargefield(esc_data,'Kurzbiographie','kurzbio'))

        file.write('</dl>')


        # Partner-Fragebogen
        fams = data['partner']
        if not fams:
            fams = [{}]
        for famnum, famdat in enumerate(fams):
            esc_famdat = TransformDict(famdat,f)

            part = 'part%d' % (famnum +1)

            file.write('<h2>Heirat oder Partnerschaft</h2>')
            file.write('<dl>')
            for desc, val in (  ('Nachnamen d. P.',   'nachname'),
                                ('Vornamen d. P.',    'vorname'),
                                ('Titel d. P.',       'title'),
                                ('Geburtsdatum d. P.','gebdat'),
                                ('Geburtsort d. P.',  'gebort')):
                file.write(mkfield(esc_famdat,desc,val,part))

            for desc, val in (  ('Datum der Heirat',   'dat'),
                                ('Ort der Heirat', 'ort') ):
                file.write(mkfield(esc_famdat,desc,val,part+'heirat'))

            for desc, val in (  ('Sterbedatum d. P.', 'sterbdat'),
                                ('Sterbeort d. P.',   'sterbort')):
                file.write(mkfield(esc_famdat,desc,val,part))

            file.write(mklargefield(esc_famdat,'Kurzbiographie d. P.','kurzbio',part))

            for desc, val in (  ('Vollst. Name des Vaters d. P.', 'vatername'),
                                ('Geburtsdatum des Vaters d. P.', 'vatergebdat'),
                                ('Geburtsort des Vaters d. P.',   'vatergebort'),
                                ('Vollst. Name der Mutter d. P.', 'muttername'),
                                ('Geburtsdatum der Mutter d. P.', 'muttergebdat'),
                                ('Geburtsort der Mutter d. P.',   'muttergebort')):
                file.write(mkfield(esc_famdat,desc,val,part))
            file.write('</dl>')

        # Kinder

        if data['kinder']:
            file.write('<h2>Kinder</h2>')

        for childcounter, childdata in enumerate(data['kinder']):
            esc_childdata = TransformDict(childdata,f)

            ordinals = ["Erstes","Zweites","Drittes","Viertes","Fünftes","Sechstes","Siebtes",
                        "Achtes","Neuntes","Zehntes","Elftes","Zwölftes","Dreizehntes","Vierzehntes"]

            file.write('<h3>%s Kind</h3>'% ordinals[childcounter])
            file.write('<dl>')
            for desc, val in (  ('Nachnamen',   'nachname'),
                                ('Vornamen',    'vorname'),
                                ('Titel',       'title'),
                                ('Geburtsdatum','gebdat'),
                                ('Geburtsort',  'gebort'),
                                ('Sterbedatum', 'sterbdat'),
                                ('Sterbeort',   'sterbort'),
                                ('Bemerkung',   'bemerkung') ):
                file.write(mkfield(esc_childdata,desc,val,"kind"+str(childcounter+1)))
            file.write('</dl>')

        # Adresse etc.

        if not data['tot']:
            file.write('<h2>Adresse</h2><dl>')
            if intern:
                for desc, val in (  ('Veröffentlichung im Buch',      'bvoe'),
                                    ('Veröffentlichung online',       'ovoe'),
                                    ('Interesse am Buch',             'bint'),
                                    ('Adressenliste: Aufnahme',       'addrla'),
                                    ('Adressenliste: Erhalt',         'addrle')):
                    file.write(mkfield(esc_data,desc,val))
            else:
                for desc, val in (  ('Veröffentlichung im Buch',      'bvoe'),
                                    ('Veröffentlichung online',       'ovoe'),
                                    ('Interesse am Buch',             'bint'),
                                    ('Adressenliste: Aufnahme',       'addrla'),
                                    ('Adressenliste: Erhalt',         'addrle')):
                    file.write(mkinfo(esc_data,desc,val))
            file.write(mklargefield(esc_data,'Adresse','adresse'))
            file.write(mkfield(esc_data,'e-Mail','email'))
            file.write('</dl>')

        file.write('<input type="submit" value="Absenden"/>')
        file.write('</form></body></html>\n')       
        file.close()

    def write_latex(self, data, num):
        filename = "%s.tex" % num
        file = open('%s/intern/%s'%(self.dir_name,filename), 'w')
        self.tex_files.append(filename)

        if data['ovoe']:
            filename = "%s.pdf" % num
            self.public_links.append(filename)

        # LaTeX-Header
        file.write('''\\documentclass[a4paper,DIV15,halfparskip,10pt]{scrbook}
                      \\newcommand{\\nummer}{%(nummer)s}
                      \\usepackage{fragebogen}
                      \\begin{document}
                      \\ihead{Daten: %(today)s}
        ''' % {'today': le(str(datetime.datetime.today())), 'nummer': numform(num) })
        # file.write('\\include{fragebogen-vorw}\n')

        def f(s):
            if s is None:
                return '\\nix'
            else:
                return '\\val{%s}' % le(s)

        #file.write('\\renewcommand{\\nachfolgenummer}{%s}\n' % numform(num))

        file.write('''
    	\\renewcommand{\\nachname}{%(nachname)s}
        \\renewcommand{\\vorname}{%(vorname)s}
        \\renewcommand{\\titel}{%(title)s}
        \\renewcommand{\\gebdat}{%(gebdat)s}
        \\renewcommand{\\gebort}{%(gebort)s}
        \\renewcommand{\\sterbdat}{%(sterbdat)s}
        \\renewcommand{\\sterbort}{%(sterbort)s}
        \\renewcommand{\\kurzbio}{%(kurzbio)s}''' % TransformDict(data,f))

        file.write('\\input{fragebogen-haupt}\\pagebreak\n')

        # Partner-Fragebogen

        for famdat in data['partner']:

            #file.write('%% %s\n' % event.get_description())
            file.write('''
            \\renewcommand{\\heiratdat}{%(dat)s}
            \\renewcommand{\\heiratort}{%(ort)s}
            \\renewcommand{\\partnachname}{%(nachname)s}
            \\renewcommand{\\partvorname}{%(vorname)s}
            \\renewcommand{\\parttitel}{%(title)s}
            \\renewcommand{\\partgebdat}{%(gebdat)s}
            \\renewcommand{\\partgebort}{%(gebort)s}
            \\renewcommand{\\partsterbdat}{%(sterbdat)s}
            \\renewcommand{\\partsterbort}{%(sterbort)s}
            \\renewcommand{\\partkurzbio}{%(kurzbio)s}
            \\renewcommand{\\partvatername}{%(vatername)s}
            \\renewcommand{\\partvatergebdat}{%(vatergebdat)s}
            \\renewcommand{\\partvatergebort}{%(vatergebort)s}
            \\renewcommand{\\partmuttername}{%(muttername)s}
            \\renewcommand{\\partmuttergebdat}{%(muttergebdat)s}
            \\renewcommand{\\partmuttergebort}{%(muttergebort)s}
            ''' % TransformDict(famdat,f))
            
            file.write('\\include{fragebogen-partner}\n')

        # Kinder

        if data['kinder']:
                file.write('\\input{fragebogen-kind.tex}\n')

        for childcounter, childdata in enumerate(data['kinder']):
            ordinals = ["Erstes","Zweites","Drittes","Viertes","Fünftes","Sechstes","Siebtes",
                        "Achtes","Neuntes","Zehntes","Elftes","Zwölftes","Dreizehntes","Vierzehntes"]

            template = '\\kind{%(num)s}{%(ord)s}{%%(nachname)s}{%%(vorname)s}{%%(gebdat)s}{%%(gebort)s}{%%(sterbdat)s}{%%(sterbort)s}{%%(bemerkung)s}\n' % {'num': childcounter+1, 'ord': ordinals[childcounter]}

            file.write(template % TransformDict(childdata,f))
                

        file.write('\\include{fragebogen-interesse}\n')
        file.write('\\end{document}\n')       

        file.close()
    
    def get_data(self, person, num):

        data = {}
        data['num'] = num
        data['nummer'] = numform(num)
        data['generation'] = numgeneration(num)

        for field in ('bvoe', 'ovoe'):
            data[field] = None
        for field in ('bint', 'addrla', 'addrle', 'adresse','email'):
            data[field] = u""
        for attrib in person.get_attribute_list():
            if   attrib.type == u"Buchveröffentlichung":
                if attrib.value == "Ja":
                    data['bvoe'] = True
                elif attrib.value == "Nein":
                    data['bvoe'] = False
            elif attrib.type == u"Onlineveröffentlichung":
                if attrib.value == "Ja":
                    data['ovoe'] = True
            elif attrib.type == u"Buchinteresse":
                data['bint'] = attrib.get_value()
            elif attrib.type == u"Adresslisten-Aufnahme":
                data['addrla'] = attrib.get_value()
            elif attrib.type == u"Adresslisten-Erhalt":
                data['addrle'] = attrib.get_value()
        for url in person.get_url_list():
            if url.type == "E-Mail":
                if data['email']:
                    data['email'] += ", "
                data['email'] += url.get_path()
            elif url.type == "Homepage":
                pass
            else:
                print "Unknown Internet field, type %s, value %s" % \
                            (str(url.type), url.get_path())
        for address in person.get_address_list():
                fax = ''
                notelist = address.get_note_list()
                for notehandle in notelist:
                    note = self.database.get_note_from_handle(notehandle)
                    fax += note.get()
                data['adresse'] = u'%s\n%s %s\n%s\n%s\n%s' % (
                            address.get_street(),
                            address.get_postal_code(),
                            address.get_city(),
                            address.get_country(),
                            address.get_phone(),
                            fax)
        
        data['geschl'] = person.get_gender()

        pf = PersonFormatter(self.database,person)
        data['obj'] = person
        data['name'] = pf['full_name']
        data['kurzname'] = pf['short_name']
        data['nachname'] = pf['full_second_name']
        data['vorname'] = pf['full_first_name']
        data['kurznachname'] = pf['main_second_name']
        data['kurzvorname'] = pf['main_first_name']
        for k in 'title gebdat gebort sterbdat sterbort kurzbio kurzbiographie redbem intnotiz quellen werke redaktionsnotiz events tot mindj gjahr tjahr prefix'.split():
            if k in pf:
                data[k] = pf[k]

        # Dead people always agree...
        if data['tot']:
            data['ovoe'] = True

        data['buchok'] = False
        if data['tot']:
            data['buchok'] = True

        if data['bvoe']:
            data['buchok'] = True

        if data['mindj']:
            parents_handle = person.get_main_parents_family_handle()
            family = self.database.get_family_from_handle(parents_handle)
            for handle in (family.get_father_handle(), family.get_mother_handle()):
                parent = self.database.get_person_from_handle(handle)
                if parent:
                    for attrib in parent.get_attribute_list():
                        if attrib.type == u"Buchveröffentlichung" and attrib.value == "Ja":
                            data['buchok'] = True

        main_family_handle = person.get_main_parents_family_handle()
        main_family = self.database.get_family_from_handle(main_family_handle)

        if main_family:
            father_handle = main_family.get_father_handle()
            father = self.database.get_person_from_handle(father_handle)

            if father:
                data['fatherobj'] = father
                fpf = PersonFormatter(self.database,father)
                for k in ['gebdat','gebort','plaingebdat','plaingebort','geborthandle','sterborthandle']:
                    if k in fpf:
                        data['vater'+k] = fpf[k]
                data['vatername'] = fpf['full_name']
                data['vaterkurznachname'] = fpf['main_second_name']
                data['vaterkurzvorname'] = fpf['main_first_name']

            mother_handle = main_family.get_mother_handle()
            mother = self.database.get_person_from_handle(mother_handle)

            if mother:
                data['motherobj'] = mother
                mpf = PersonFormatter(self.database,mother)
                for k in ['gebdat','gebort','plaingebdat','plaingebort','geborthandle','sterborthandle']:
                    if k in mpf:
                        data['mutter'+k] = mpf[k]
                data['muttername'] = mpf['full_name']
                data['mutterkurznachname'] = mpf['main_second_name']
                data['mutterkurzvorname'] = mpf['main_first_name']

        data['partner'] = []
        data['kinder'] = []
        family_handles = person.get_family_handle_list()
        multiple_partners = len(family_handles) > 1
        for family_handle in family_handles:
            family = self.database.get_family_from_handle(family_handle)

            famdat = {}
            famdat['events'] = []
            for field in ('adresse','email'):
                famdat[field] = ""

            for event_handle_ref in family.get_event_ref_list():
                event = self.database.get_event_from_handle(event_handle_ref.ref)
                if event:
                    if event.get_type() in (gen.lib.EventType.MARRIAGE,):
                        thisdat = dd.display(event.get_date_object())
                        place_handle = event.get_place_handle()
                        place = self.database.get_place_from_handle(place_handle)
                        thisplace = ''
                        if place:
                            thisplace = place.get_title()
                        
                        if event.get_description():
                            thisdat += " (%s)" % event.get_description()
                            if thisplace:
                                thisplace += " (%s)" % event.get_description()

                        if 'dat' in famdat:
                            famdat['dat'] += ", " + thisdat
                        else:
                            famdat['dat'] = thisdat
                        if thisplace:
                            if 'ort' in famdat:
                                famdat['ort'] += ", " + thisplace
                            else:
                                famdat['ort'] = thisplace

                    elif event.get_type() in (gen.lib.EventType.DIVORCE,):
                        thisdat = dd.display(event.get_date_object())
                        place_handle = event.get_place_handle()
                        place = self.database.get_place_from_handle(place_handle)
                        thisplace = ''
                        if place:
                            thisplace = place.get_title()
                            thisplace += " (Scheidung)"

                        thisdat += " (Scheidung)"

                        if 'dat' in famdat:
                            famdat['dat'] += ", " + thisdat
                        else:
                            famdat['dat'] = thisdat
                        if thisplace:
                            if 'ort' in famdat:
                                famdat['ort'] += ", " + thisplace
                            else:
                                famdat['ort'] = thisplace

            spouse_handle = ReportUtils.find_spouse(person,family)
            spouse_nick = ""
            famdat['geschl'] = gen.lib.Person.UNKNOWN
            if spouse_handle:
                spouse = self.database.get_person_from_handle(spouse_handle)
                famdat['spouseobj'] = spouse
                famdat['geschl'] = spouse.get_gender()
                
                spf = PersonFormatter(self.database,spouse)

                famdat['name'] = spf['full_name']
                famdat['kurzname'] = spf['short_name']
                famdat['nachname'] = spf['full_second_name']
                famdat['vorname'] = spf['full_first_name']
                famdat['kurznachname'] = spf['main_second_name']
                famdat['kurzvorname'] = spf['main_first_name']
                spouse_nick = spf['full_first_name']
                for k in 'title gebdat gebort sterbdat sterbort kurzbio kurzbiographie redbem intnotiz quellen werke redaktionsnotiz tot events gjahr tjahr prefix'.split():
                    if k in spf:
                        famdat[k] = spf[k]

                # Partnereltern

                par_family_handle = spouse.get_main_parents_family_handle()
                par_family = self.database.get_family_from_handle(par_family_handle)

                if par_family:
                    father_handle = par_family.get_father_handle()
                    father = self.database.get_person_from_handle(father_handle)

                    if father:
                        famdat['spousefatherobj'] = father
                        fpf = PersonFormatter(self.database,father)
                        for k in ['gebdat','gebort','plaingebdat','plaingebort','geborthandle','sterborthandle']:
                            if k in fpf:
                                famdat['vater'+k] = fpf[k]
                        famdat['vatername'] = fpf['full_name']
                        famdat['vaterkurznachname'] = fpf['main_second_name']
                        famdat['vaterkurzvorname'] = fpf['main_first_name']

                    mother_handle = par_family.get_mother_handle()
                    mother = self.database.get_person_from_handle(mother_handle)

                    if mother:
                        famdat['spousemotherobj'] = mother
                        mpf = PersonFormatter(self.database,mother)
                        for k in ['gebdat','gebort','plaingebdat','plaingebort','geborthandle','sterborthandle']:
                            if k in mpf:
                                famdat['mutter'+k] = mpf[k]
                        famdat['muttername'] = mpf['full_name']
                        famdat['mutterkurznachname'] = mpf['main_second_name']
                        famdat['mutterkurzvorname'] = mpf['main_first_name']

                for url in spouse.get_url_list():
                    if url.type == "E-Mail":
                        if famdat['email']:
                            famdat['email'] += ", "
                        famdat['email'] += url.get_path()
                    elif url.type == "Homepage":
                        pass
                    else:
                        print "Unknown Internet field, type %s, value %s" % \
                                    (str(url.type), url.get_path())
                for address in spouse.get_address_list():
                        fax = ''
                        notelist = address.get_note_list()
                        for notehandle in notelist:
                            note = self.database.get_note_from_handle(notehandle)
                            fax += note.get()
                        famdat['adresse'] = u'%s\n%s %s\n%s\n%s\n%s' % (
                                    address.get_street(),
                                    address.get_postal_code(),
                                    address.get_city(),
                                    address.get_country(),
                                    address.get_phone(),
                                    fax)
            # Kinder

            famdat['kinder'] = []
            child_list = family.get_child_ref_list()
            for child_ref in child_list:
                c = len(data['kinder']) 

                childdat = {}
                child = self.database.get_person_from_handle(child_ref.ref)

                childnum = num + cchars[c]
                if childnum in self.persons.doubles:
                    childdat['num'] = self.persons.doubles[childnum]
                else:
                    childdat['num'] = childnum
                childdat['nummer'] = numform(childdat['num'])
                
                childdat['geschl'] = child.get_gender()


                cpf = PersonFormatter(self.database, child)
                childdat['name'] = cpf['full_name']
                childdat['kindname'] = cpf['child_name']
                childdat['kurzname'] = cpf['short_name']
                childdat['nachname'] = cpf['child_second_name']
                childdat['vorname'] = cpf['full_first_name']
                childdat['kurznachname'] = cpf['main_second_name']
                childdat['kurzvorname'] = cpf['main_first_name']
                for k in 'title gebdat gebort sterbdat sterbort tot mindj'.split() + \
                        'plaingebdat plaingebort plainsterbdat plainsterbort'.split():
                    if k in cpf:
                        childdat[k] = cpf[k]
                
                childdat['buchok'] = False
                for attrib in child.get_attribute_list():
                    if   attrib.type == u"Buchveröffentlichung":
                        if attrib.value == "Ja":
                            childdat['buchok'] = True
                if childdat['tot']:
                    childdat['buchok'] = True
                if childdat['mindj'] and data['buchok']:
                    childdat['buchok'] = True

                bemerkungen = []
                if spouse_handle and multiple_partners:
                    bemerkungen.append("mit %s" %spouse_nick)

                father_rel = child_ref.get_father_relation()
                mother_rel = child_ref.get_mother_relation()
                if mother_rel == gen.lib.ChildRefType.FOSTER and father_rel == gen.lib.ChildRefType.FOSTER:
                    bemerkungen.append("Pflegekind")
                else:
                    if father_rel == gen.lib.ChildRefType.ADOPTED:
                        bemerkungen.append("vom Vater adoptiert")
                    elif father_rel != gen.lib.ChildRefType.BIRTH:
                        bemerkungen.append("vom Vater kein leibliches Kind (genauere Angabe erforderlich)")
                    if mother_rel == gen.lib.ChildRefType.ADOPTED:
                        bemerkungen.append("von der Mutter adoptiert")
                    elif mother_rel != gen.lib.ChildRefType.BIRTH:
                        bemerkungen.append("von der Mutter kein leibliches Kind (genauere Angabe erforderlich)")

                childdat['bemerkung'] = ", ".join(bemerkungen)

                childdat['obj'] = child_ref.ref

                famdat['kinder'].append(childdat)
                data['kinder'].append(childdat)

            famdat['num'] = data['num']
            famdat['nummer'] = data['nummer']
            data['partner'].append(famdat)                

        # Hier Kinder Sortieren!
        bd_sort = Sort.Sort(self.database).by_birthdate
        data['kinder'].sort(bd_sort, lambda x: x['obj'])
        for dat in data['kinder']: del dat['obj']


        return data


    def add_styles(self,doc):
        pass

class RiehmExtract(RiehmBase):
    def __init__(self,database,options):
        """
        Creates RiehmWebSite object that produces the report.
        
        The arguments are:

        database        - the GRAMPS database instance
        person          - currently selected person
        options   - instance of the Options class for this report
        """

        print "RiehmExtract init"

        self.database = database
        self.options = options

        menu = options.menu
        pid = menu.get_option_by_name('pid').get_value()
        self.start_person = database.get_person_from_gramps_id(pid)
        assert self.start_person

        self.target_file = menu.get_option_by_name('of').get_value()
        self.selection = menu.get_option_by_name('selection').get_value()
        self.source_path = menu.get_option_by_name('sd').get_value()
        self.intern = menu.get_option_by_name('intern').get_value()

        # Hack?

        self.doc = NoDoc()

    def begin_report(self):
        pass

    def write_report(self):
        self.progress = ProgressMeter(_("Generate RiehmWebSites"),'')
        try:
            self.write_book_pdf(self.target_file,self.intern,False)
        finally: 
            self.progress.close()

    def end_report(self):
        pass

class RiehmKML(RiehmBase):
    def __init__(self,database,options):
        """
        Creates RiehmWebSite object that produces the report.
        
        The arguments are:

        database        - the GRAMPS database instance
        person          - currently selected person
        options   - instance of the Options class for this report
        """

        print "RiehmExtract init"

        self.database = database
        self.options = options

        menu = options.menu
        pid = menu.get_option_by_name('pid').get_value()
        self.start_person = database.get_person_from_gramps_id(pid)
        assert self.start_person

        self.target_file = menu.get_option_by_name('of').get_value()
        self.selection = menu.get_option_by_name('selection').get_value()
        self.source_path = menu.get_option_by_name('sd').get_value()
        self.intern = menu.get_option_by_name('intern').get_value()

        # Hack?

        self.doc = NoDoc()

    def begin_report(self):
        pass

    def write_report(self):
        self.progress = ProgressMeter(_("Generate RiehmWebSites"),'')
        try:
            self.write_kml(self.target_file,self.intern)
        finally: 
            self.progress.close()

    def end_report(self):
        pass

class RiehmWebSite(RiehmBase):
    def __init__(self,database,options):
        """
        Creates RiehmWebSite object that produces the report.
        
        The arguments are:

        database        - the GRAMPS database instance
        person          - currently selected person
        options   - instance of the Options class for this report

        This report needs the following parameters (class variables)
        that come in the options class.
        
        filter
        od
        genpdf
        """

        print "RiehmWebSite init"
        self.database = database
        self.options = options

        menu = options.menu
        pid = menu.get_option_by_name('pid').get_value()
        self.start_person = database.get_person_from_gramps_id(pid)
        assert self.start_person

        self.target_path = menu.get_option_by_name('od').get_value()
        self.genpdf = menu.get_option_by_name('genpdf').get_value()
        self.genbook = menu.get_option_by_name('genbook').get_value()
        self.genhtml = menu.get_option_by_name('genhtml').get_value()
        self.genevents = menu.get_option_by_name('genevents').get_value()
        self.source_path = menu.get_option_by_name('sd').get_value()
        self.selection = None

        # Hack?

        self.doc = NoDoc()

    def begin_report(self):
        pass

    def end_report(self):
        pass

    def write_report(self):
        print "RiehmWebSite write report"
        dir_name = self.target_path
        if dir_name == None:
            dir_name = os.getcwd()
        elif not os.path.isdir(dir_name):
            parent_dir = os.path.dirname(dir_name)
            if not os.path.isdir(parent_dir):
                ErrorDialog(_("Neither %s nor %s are directories") % \
                    (dir_name,parent_dir))
                return
            else:
                try:
                    os.mkdir(dir_name)
                except IOError, value:
                    ErrorDialog(_("Could not create the directory: %s") % \
                        dir_name + "\n" + value[1])
                    return
                except:
                    ErrorDialog(_("Could not create the directory: %s") % \
                        dir_name)
                    return

        if not os.path.isdir(dir_name+"/intern"):
            parent_dir = os.path.dirname(dir_name+"/intern")
            if not os.path.isdir(parent_dir):
                ErrorDialog(_("Neither %s nor %s are directories") % \
                    (dir_name,parent_dir))
                return
            else:
                try:
                    os.mkdir(dir_name+"/intern")
                except IOError, value:
                    ErrorDialog(_("Could not create the directory: %s") % \
                        dir_name + "/intern\n" + value[1])
                    return
                except:
                    ErrorDialog(_("Could not create the directory: %s") % \
                        dir_name+"/intern")
                    return
        self.dir_name = dir_name                
            
        self.progress = ProgressMeter(_("Generate RiehmWebSites"),'')
        self.tex_files = []
        self.public_links = []
        try:
            self.persons = Descendants(self.database,self.start_person)    
            

            if self.genhtml:
                if self.genpdf:
                    self.progress.set_pass(_('Writing latex and HTML files'),len(self.persons))
                else:
                    self.progress.set_pass(_('Writing HTML files'),len(self.persons))
                
                for person_handle, num in self.persons:
                    if num in self.persons.doubles:
                        # Skip Doubles
                        pass
                    else:
                        person = self.database.get_person_from_handle(person_handle)
                        person_data = self.get_data(person,num)
                        #self.write_json(person_data,num)
                        if self.genpdf:
                            self.write_latex(person_data,num)
                        self.write_html(person_data,num,False)
                        self.write_html(person_data,num,True)
                    self.progress.step()

                self.progress.set_pass(_('Writing Index'),2)
                self.write_index(True)
                self.write_index(False)
                self.progress.step()
                self.write_list()
                self.progress.step()

            if self.genevents:
                self.progress.set_pass(_('Writing eventlist'),1)
                self.write_eventlist()
                self.progress.step()

            self.progress.set_pass(_('Writing e-Mail-List'),1)
            self.write_emaillist()
            self.progress.step()

            copyfiles = [   "fragebogen-brief.tex",
                            "fragebogen-haupt.tex",
                            "fragebogen-interesse.tex",
                            "fragebogen-kind.tex",
                            "fragebogen-leer.tex",
                            "fragebogen-partner.tex",
                            "fragebogen-vorw.tex",
                            "fragebogen.sty",
                            "isaak-pkgs.sty",
                            "numind.sty",
                            "ebenda.sty",
                            "Isaak+Charlotte.jpg",
                            "blanko-intern.html",
                            "burial.tfm",
                            "burial.enc",
                            "burial.sty",
                            "t1burial.fd",
                            "ManPictograms.ttf",
                            "ManPictograms.tfm",
                            "ManPictograms.enc",
                            "ManPictograms.sty",
                            "T1ManPictograms.fd",
                            ]
            
            copyfiles_public = [
                            "send.py",
                            "blanko.html",
                            ]

            dir = self.dir_name
            intern = self.dir_name + "/intern"

            self.progress.set_pass(_('Copying source files'),
                                   len(copyfiles + copyfiles_public))
            for file in copyfiles:
                   shutil.copy(self.source_path + "/" + file, intern)
                   self.progress.step()
            for file in copyfiles_public:
                   shutil.copy(self.source_path + "/" + file, dir)
                   self.progress.step()

            # files = map (lambda (p,n): "%s.tex"%n, persons)
            if False: # Keine fragebogen bauen
                self.tex_files.extend([  "fragebogen-brief.tex",
                                         "fragebogen-leer.tex",
                             ])
                self.public_links.extend([ "fragebogen-leer.pdf",
                                           "fragebogen-brief.pdf"])
                self.progress.set_pass(_('Running pdflatex'),len(self.tex_files))
                for file in self.tex_files:
                    subprocess.call(["pdflatex",file], cwd=intern)
                    self.progress.step()

            if self.genbook:
                self.write_book_pdf("%s/intern/%s"%(self.dir_name,'book.pdf'),False,False)
                self.write_book_pdf("%s/intern/%s"%(self.dir_name,'data.pdf'),True,False)
                self.write_book_pdf("%s/intern/%s"%(self.dir_name,'data_draft.pdf'),True,True)

            self.progress.set_pass(_("Generating address list"), 1)
            self.write_csv("%s/intern/adressen.csv" % self.dir_name)

            self.progress.set_pass(_("Linking files"), len(self.public_links))
            for file in self.public_links:
                    if not os.path.islink(dir+"/"+file):
                        print "symlinking %s"%file
                        os.symlink("intern/"+file, dir+"/"+file)
            
        finally: 
            self.progress.close()

class RiehmWebSiteOptions(MenuReportOptions):

    """
    Defines options and provides handling interface.
    """

    def __init__(self, name, dbase):
        MenuReportOptions.__init__(self, name, dbase)

    def add_menu_options(self,menu):

        category_name = _("RiehmWebSite Options")

        pid = PersonOption(_("Center Person"))
        pid.set_help(_("The center person for the tree"))
        menu.add_option(category_name, "pid", pid)

        genpdf = BooleanOption(_('Generate PDFs'), False)
        menu.add_option(category_name, "genpdf", genpdf)

        genbook = BooleanOption(_('Generate Book'), True)
        menu.add_option(category_name, "genbook", genbook)

        genhtml = BooleanOption(_('Generate HTML'), True)
        menu.add_option(category_name, "genhtml", genhtml)

        genevents = BooleanOption(_('Generate Event files'), True)
        menu.add_option(category_name, "genevents", genevents)

        od = DestinationOption(_('Output directory'), './')
        od.set_directory_entry(True)
        menu.add_option(category_name, "od", od)

        sd = DestinationOption(_('Source directory'), '/home/jojo/projekte/ahnenforschung/Isaak-Buch/fragebogen')
        sd.set_directory_entry(True)
        menu.add_option(category_name, "sd", sd)

    #------------------------------------------------------------------------
    #
    # Callback functions from the dialog
    #
    #------------------------------------------------------------------------
    def make_default_style(self,default_style):
        """Make the default output style for the Web Pages Report."""
        pass

class RiehmExtractOptions(MenuReportOptions):

    """
    Defines options and provides handling interface.
    """

    def __init__(self, name, dbase):
        MenuReportOptions.__init__(self, name, dbase)

    def add_menu_options(self,menu):

        category_name = _("RiehmExtract Options")

        pid = PersonOption(_("Center Person"))
        pid.set_help(_("The center person for the tree"))
        menu.add_option(category_name, "pid", pid)

        selection = StringOption(_('Selection '), '1*')
        menu.add_option(category_name, "selection", selection)

        intern = BooleanOption(_('Include internal data'), False)
        menu.add_option(category_name, "intern", intern)

        sd = DestinationOption(_('Source directory'), '/home/jojo/projekte/ahnenforschung/Isaak-Buch/fragebogen')
        sd.set_directory_entry(True)
        menu.add_option(category_name, "sd", sd)

        of = DestinationOption(_('Output file'), './')
        of.set_extension('pdf')
        menu.add_option(category_name, "of", of)

    #------------------------------------------------------------------------
    #
    # Callback functions from the dialog
    #
    #------------------------------------------------------------------------
    def make_default_style(self,default_style):
        """Make the default output style for the Web Pages Report."""
        pass

class RiehmKMLOptions(MenuReportOptions):

    """
    Defines options and provides handling interface.
    """

    def __init__(self, name, dbase):
        MenuReportOptions.__init__(self, name, dbase)

    def add_menu_options(self,menu):

        category_name = _("RiehmKML Options")

        pid = PersonOption(_("Center Person"))
        pid.set_help(_("The center person for the tree"))
        menu.add_option(category_name, "pid", pid)

        selection = StringOption(_('Selection '), '1*')
        menu.add_option(category_name, "selection", selection)

        intern = BooleanOption(_('Include internal data'), False)
        menu.add_option(category_name, "intern", intern)

        sd = DestinationOption(_('Source directory'), '/home/jojo/projekte/ahnenforschung/Isaak-Buch/fragebogen')
        sd.set_directory_entry(True)
        menu.add_option(category_name, "sd", sd)

        of = DestinationOption(_('Output file'), './')
        of.set_extension('kml')
        menu.add_option(category_name, "of", of)

    #------------------------------------------------------------------------
    #
    # Callback functions from the dialog
    #
    #------------------------------------------------------------------------
    def make_default_style(self,default_style):
        """Make the default output style for the Web Pages Report."""
        pass

#------------------------------------------------------------------------
#
# Empty class to keep the BaseDoc-targeted format happy
#
#------------------------------------------------------------------------
class EmptyDoc:
    def __init__(self,styles,type,template,orientation,source=None):
        pass

    def init(self):
        pass

#-------------------------------------------------------------------------
