{-# LANGUAGE PatternGuards #-}

import Text.ParserCombinators.Parsec
import System.Environment
import Data.Char
import Data.List
import Data.Maybe
import Control.Monad
import Graphics.Rendering.Cairo


first_year = 1795
last_year = 2010

width, height, linePos :: Num a => a
width = 3024
height = 10000
linePos = 50

parse_line line | (num:date:names) <- words line,
		  all (isDigit) date
                = Nothing
                | (num:date:names) <- words line,
		  any (isAlpha) date
                = Nothing
                | (num:date:names) <- words line
                = Just (parseDate date,num, unwords names)
                | otherwise = error $ "Could not parse " ++ line

parseDate :: String -> (Int, Int, Int)
parseDate str = case parse parser "Datum" str of
	Left err -> error $ "Could not parse date " ++ str ++ ": " ++ show err
	Right ret -> ret
  where parser = do
		days <- many digit
		char '.'
		months <- many digit
		char '.'
		years <- count 4 digit
		return (read years, read months, read days)

--distort = id
distort n = n*n

dateToFloat (y,m,d) =  distort $
                       (fromIntegral (y-first_year) +
			fromIntegral (m-1) / 12 + 
			fromIntegral (d-1) / (356)
                       ) / fromIntegral (last_year - first_year)
showDate (y,m,d) = show d ++ "." ++ show m ++ "."

drawTimeline = do
	setLineWidth 1
	setSourceRGB 1 1 1
	paint
	setSourceRGB 0 0 0
	moveTo linePos linePos
	lineTo (linePos) (height - linePos)
	stroke

drawYears = forM_ [first_year..last_year-1] $ \year -> do
	te <- textExtents (show year)
	moveTo linePos (yearToVPos (year,7,1)) 
	relMoveTo (-textExtentsWidth te - 8) 0
	showText (show year)

drawYearsLines = forM_ [first_year..last_year] $ \year -> do
	te <- textExtents (show year)
	moveTo linePos (yearToVPos (year,1,1)) 
	relLineTo (-textExtentsWidth te - 8) 0
	stroke

drawEntry (w,s) (date, num, name) = do
	let text = " * " ++ showDate date ++ " " ++ name ++ " ["++num++"]"
	te <- textExtents text
	let vPos = yearToVPos date
	let hPos = if vPos < s then w
                               else 8
	moveTo linePos (yearToVPos date) 
	relLineTo hPos 0
	relLineTo 0 (-textExtentsHeight te - 2)
	stroke
	moveTo linePos (yearToVPos date) 
	relMoveTo hPos (-2)
	showText text

	return (hPos + textExtentsWidth te + 8, vPos + textExtentsHeight te + 2)

yearToVPos d = linePos + dateToFloat d * (height - 2*linePos)

render entries = do
	drawTimeline
	drawYears
	drawYearsLines
	foldM_ drawEntry (0,0) entries

main = do
	--[inFile] <- getArgs
	let inFile = "/home/jojo/remote-websites/riehm.name/fragebogen/intern/births.dat"
	let basename = "timeline"

	entries <- (sort . mapMaybe parse_line . lines) `fmap` readFile inFile

	withImageSurface FormatRGB24 width height $ \sur -> do
		renderWith sur (render entries)
		surfaceWriteToPNG sur (basename ++ ".png")

